@@ -5,8 +5,6 @@
  *
  * Part:        sock.c include file.
  *
- * Version:     $Id: sock.h,v 1.1.16 2009/02/14 03:25:07 acassen Exp $
- *
  * Authors:     Alexandre Cassen, <acassen@linux-vs.org>
  *
  *              This program is distributed in the hope that it will be useful,
@@ -19,7 +17,7 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #ifndef _SOCK_H
@@ -41,9 +39,11 @@ typedef struct {
 	int		status;
 	int		lock;
 	char		*buffer;
-	char		*extracted;
+	const char	*extracted;
 	int		size;
 	int		total_size;
+	ssize_t		content_len;
+	ssize_t		rx_bytes;
 } SOCK;
 
 /* global vars exported */
