@@ -43,14 +43,11 @@
 #include "signals.h"
 #include "assert_debug.h"
 
-/* RFC5881 section 4 */
-#define	BFD_MIN_PORT	49152
-#define	BFD_MAX_PORT	65535
-
+/* Locals */
 static int bfd_send_packet(int, bfdpkt_t *, bool);
 static void bfd_sender_schedule(bfd_t *);
 
-static void bfd_state_down(bfd_t *, u_char diag);
+static void bfd_state_down(bfd_t *, uint8_t diag);
 static void bfd_state_admindown(bfd_t *);
 static void bfd_state_up(bfd_t *);
 static void bfd_dump_timers(FILE *fp, bfd_t *);
@@ -78,7 +75,7 @@ thread_time_to_wakeup(thread_ref_t thread)
 }
 
 /* Sends one BFD control packet and reschedules itself if needed */
-static int
+static void
 bfd_sender_thread(thread_ref_t thread)
 {
 	bfd_t *bfd;
@@ -107,8 +104,6 @@ bfd_sender_thread(thread_ref_t thread)
 	/* Schedule next run if not called as an event thread */
 	if (thread->type != THREAD_EVENT)
 		bfd_sender_schedule(bfd);
-
-	return 0;
 }
 
 /* Schedules bfd_sender_thread to run in local_tx_intv minus applied jitter */
@@ -230,7 +225,7 @@ bfd_sender_discard(bfd_t *bfd)
  */
 
 /* Marks session as down because of Control Detection Time Expiration */
-static int
+static void
 bfd_expire_thread(thread_ref_t thread)
 {
 	bfd_t *bfd;
@@ -268,8 +263,6 @@ bfd_expire_thread(thread_ref_t thread)
 	 */
 	bfd->remote_discr = 0;
 	bfd_state_down(bfd, BFD_DIAG_EXPIRED);
-
-	return 0;
 }
 
 /* Schedules bfd_expire_thread to run in local_detect_time */
@@ -365,7 +358,7 @@ bfd_expire_discard(bfd_t *bfd)
  */
 
 /* Resets BFD session to initial state */
-static int
+static void
 bfd_reset_thread(thread_ref_t thread)
 {
 	bfd_t *bfd;
@@ -379,8 +372,6 @@ bfd_reset_thread(thread_ref_t thread)
 	bfd->thread_rst = NULL;
 
 	bfd_reset_state(bfd);
-
-	return 0;
 }
 
 /* Schedules bfd_reset_thread to run in local_detect_time */
@@ -485,7 +476,7 @@ bfd_state_fall(bfd_t *bfd, bool send_event)
 
 /* Runs when BFD session state goes Down */
 static void
-bfd_state_down(bfd_t *bfd, u_char diag)
+bfd_state_down(bfd_t *bfd, uint8_t diag)
 {
 	assert(bfd);
 	assert(BFD_VALID_DIAG(diag));
@@ -895,7 +886,7 @@ bfd_receive_packet(bfdpkt_t *pkt, int fd, char *buf, ssize_t bufsz)
  */
 
 /* Runs when data is available in listening socket */
-static int
+static void
 bfd_receiver_thread(thread_ref_t thread)
 {
 	bfd_data_t *data;
@@ -921,8 +912,6 @@ bfd_receiver_thread(thread_ref_t thread)
 	data->thread_in =
 	    thread_add_read(thread->master, bfd_receiver_thread, data,
 			    fd, TIMER_NEVER, false);
-
-	return 0;
 }
 
 /*
@@ -994,8 +983,8 @@ read_local_port_range(uint32_t port_limits[2])
 	val[0] = strtol(buf, &endptr, 10);
 	if (val[0] <= 0 || val[0] == LONG_MAX || (*endptr != '\t' && *endptr != ' '))
 		return false;
-	val[1] = strtol(buf, &endptr, 10);
-	if (val[1] <= 0 || val[0] == LONG_MAX || *endptr != '\n')
+	val[1] = strtol(endptr + 1, &endptr, 10);
+	if (val[1] <= 0 || val[1] == LONG_MAX || *endptr != '\n')
 		return false;
 
 	port_limits[0] = val[0];
@@ -1012,6 +1001,7 @@ bfd_open_fd_out(bfd_t *bfd)
 	int ret;
 	uint32_t port_limits[2];
 	uint16_t orig_port, port;
+	socklen_t sockaddr_len;
 
 	assert(bfd);
 	assert(bfd->fd_out == -1);
@@ -1040,6 +1030,7 @@ bfd_open_fd_out(bfd_t *bfd)
 		}
 
 		orig_port = port = rand_intv(port_limits[0], port_limits[1]);
+		sockaddr_len = bfd->src_addr.ss_family == AF_INET ? sizeof(struct sockaddr_in) : sizeof(struct sockaddr_in6);
 		do {
 			/* Try binding socket to the address until we find one available */
 			if (bfd->src_addr.ss_family == AF_INET)
@@ -1047,8 +1038,7 @@ bfd_open_fd_out(bfd_t *bfd)
 			else
 				((struct sockaddr_in6 *)&bfd->src_addr)->sin6_port = htons(port);
 
-			ret = bind(bfd->fd_out, (struct sockaddr *) &bfd->src_addr,
-				   sizeof (struct sockaddr));
+			ret = bind(bfd->fd_out, (struct sockaddr *) &bfd->src_addr, sockaddr_len);
 
 			if (ret == -1 && errno == EADDRINUSE) {
 				/* Port already in use, try next */
@@ -1095,10 +1085,8 @@ static int
 bfd_open_fds(bfd_data_t *data)
 {
 	bfd_t *bfd;
-	element e;
 
 	assert(data);
-	assert(data->bfd);
 
 	/* Do not reopen input socket on reload */
 	if (bfd_data->fd_in == -1) {
@@ -1110,10 +1098,7 @@ bfd_open_fds(bfd_data_t *data)
 		}
 	}
 
-	for (e = LIST_HEAD(data->bfd); e; ELEMENT_NEXT(e)) {
-		bfd = ELEMENT_DATA(e);
-		assert(bfd);
-
+	list_for_each_entry(bfd, &data->bfd, e_list) {
 		if (bfd_open_fd_out(bfd)) {
 			log_message(LOG_ERR, "BFD_Instance(%s) Unable to"
 				    " open output socket, disabling instance",
@@ -1130,7 +1115,6 @@ static void
 bfd_register_workers(bfd_data_t *data)
 {
 	bfd_t *bfd;
-	element e;
 
 	assert(data);
 	assert(!data->thread_in);
@@ -1140,9 +1124,7 @@ bfd_register_workers(bfd_data_t *data)
 					  data, data->fd_in, TIMER_NEVER, false);
 
 	/* Resume or schedule threads */
-	for (e = LIST_HEAD(data->bfd); e; ELEMENT_NEXT(e)) {
-		bfd = ELEMENT_DATA(e);
-
+	list_for_each_entry(bfd, &data->bfd, e_list) {
 		/* Do not start anything if instance is in AdminDown state.
 		   Discard saved state if any */
 		if (bfd_sender_suspended(bfd)) {
@@ -1181,7 +1163,6 @@ void
 bfd_dispatcher_release(bfd_data_t *data)
 {
 	bfd_t *bfd;
-	element e;
 
 	assert(data);
 
@@ -1203,9 +1184,7 @@ bfd_dispatcher_release(bfd_data_t *data)
 
 	/* Suspend threads for possible resuming after reconfiguration */
 	set_time_now();
-	for (e = LIST_HEAD(data->bfd); e; ELEMENT_NEXT(e)) {
-		bfd = ELEMENT_DATA(e);
-
+	list_for_each_entry(bfd, &data->bfd, e_list) {
 		if (bfd_sender_scheduled(bfd))
 			bfd_sender_suspend(bfd);
 
@@ -1225,7 +1204,7 @@ bfd_dispatcher_release(bfd_data_t *data)
 }
 
 /* Starts BFD dispatcher */
-int
+void
 bfd_dispatcher_init(thread_ref_t thread)
 {
 	bfd_data_t *data;
@@ -1237,8 +1216,6 @@ bfd_dispatcher_init(thread_ref_t thread)
 		exit(EXIT_FAILURE);
 
 	bfd_register_workers(data);
-
-	return 0;
 }
 
 
