@@ -56,8 +56,8 @@ const dns_type_t DNS_TYPE[] = {
 	{0, NULL}
 };
 
-static int dns_connect_thread(thread_ref_t);
-static int dns_send_thread(thread_ref_t);
+static void dns_connect_thread(thread_ref_t);
+static void dns_send_thread(thread_ref_t);
 
 static uint16_t __attribute__ ((pure))
 dns_type_lookup(const char *label)
@@ -101,7 +101,7 @@ dns_log_message(thread_ref_t thread, int level, const char *fmt, ...)
 }
 
 static int __attribute__ ((format (printf, 3, 4)))
-dns_final(thread_ref_t thread, int error, const char *fmt, ...)
+dns_final(thread_ref_t thread, bool error, const char *fmt, ...)
 {
 	char buf[MAX_LOG_MSG];
 	va_list args;
@@ -129,7 +129,7 @@ dns_final(thread_ref_t thread, int error, const char *fmt, ...)
 				va_start(args, fmt);
 				len = vsnprintf(buf, sizeof (buf), fmt, args);
 				va_end(args);
-				if (checker->has_run && checker->retry_it >= checker->retry && !checker->has_run)
+				if (checker->has_run && checker->retry_it >= checker->retry )
 					snprintf(buf + len, sizeof(buf) - len, " after %u retries", checker->retry);
 				dns_log_message(thread, LOG_INFO, "%s", buf);
 			}
@@ -168,7 +168,7 @@ dns_final(thread_ref_t thread, int error, const char *fmt, ...)
 	return 0;
 }
 
-static int
+static void
 dns_recv_thread(thread_ref_t thread)
 {
 	unsigned long timeout;
@@ -181,8 +181,8 @@ dns_recv_thread(thread_ref_t thread)
 	dns_check_t *dns_check = CHECKER_ARG(checker);
 
 	if (thread->type == THREAD_READ_TIMEOUT) {
-		dns_final(thread, 1, "read timeout from socket");
-		return 0;
+		dns_final(thread, true, "read timeout from socket");
+		return;
 	}
 
 	timeout = timer_long(thread->sands) - timer_long(time_now);
@@ -192,10 +192,10 @@ dns_recv_thread(thread_ref_t thread)
 		if (check_EAGAIN(errno) || check_EINTR(errno)) {
 			thread_add_read(thread->master, dns_recv_thread,
 					checker, thread->u.f.fd, timeout, true);
-			return 0;
+			return;
 		}
-		dns_final(thread, 1, "failed to read socket. %s", strerror(errno));
-		return 0;
+		dns_final(thread, true, "failed to read socket; errno %d (%s)", errno, strerror(errno));
+		return;
 	}
 
 	if (ret < (ssize_t) sizeof (r_header)) {
@@ -205,7 +205,7 @@ dns_recv_thread(thread_ref_t thread)
 #endif
 		thread_add_read(thread->master, dns_recv_thread, checker,
 				thread->u.f.fd, timeout, true);
-		return 0;
+		return;
 	}
 
 	s_header = (dns_header_t *) dns_check->sbuf;
@@ -219,7 +219,7 @@ dns_recv_thread(thread_ref_t thread)
 #endif
 		thread_add_read(thread->master, dns_recv_thread, checker,
 				thread->u.f.fd, timeout, true);
-		return 0;
+		return;
 	}
 
 	flags = ntohs(r_header->flags);
@@ -231,18 +231,16 @@ dns_recv_thread(thread_ref_t thread)
 #endif
 		thread_add_read(thread->master, dns_recv_thread, checker,
 				thread->u.f.fd, timeout, true);
-		return 0;
+		return;
 	}
 
 	if ((rcode = DNS_RC(flags)) != 0) {
-		dns_final(thread, 1, "read error occurred. (rcode = %d)", rcode);
-		return 0;
+		dns_final(thread, true, "read error occurred. (rcode = %d)", rcode);
+		return;
 	}
 
 	/* success */
-	dns_final(thread, 0, NULL);
-
-	return 0;
+	dns_final(thread, false, NULL);
 }
 
 #define APPEND16(x, y) do { \
@@ -250,7 +248,7 @@ dns_recv_thread(thread_ref_t thread)
 		(x) = (uint8_t *) (x) + 2; \
 	} while(0)
 
-static int
+static void
 dns_make_query(thread_ref_t thread)
 {
 	uint16_t flags = 0;
@@ -264,7 +262,7 @@ dns_make_query(thread_ref_t thread)
 	DNS_SET_RD(flags, 1);	/* Recursion Desired */
 
 	/* coverity[dont_call] */
-	header->id = htons(random());
+	header->id = random();
 	header->flags = htons(flags);
 	header->qdcount = htons(1);
 	header->ancount = htons(0);
@@ -283,14 +281,14 @@ dns_make_query(thread_ref_t thread)
 		memcpy(p, s, n);
 		p += n;
 	}
-	*(p++) = 0;	/* Terminate the name */
+	
+	if (dns_check->name[0] != '.' || dns_check->name[1] != '\0')
+		*(p++) = 0;
 
 	APPEND16(p, dns_check->type);
 	APPEND16(p, 1);		/* IN */
 
 	dns_check->slen = (size_t)(p - (uint8_t *)header);
-
-	return 0;
 }
 
 static void
@@ -303,6 +301,10 @@ dns_send(thread_ref_t thread)
 
 	timeout = timer_long(thread->sands) - timer_long(time_now);
 
+	/* Handle time_now > thread->sands */
+	if (timeout > checker->co->connection_to)
+		timeout = 0;
+
 	ret = send(thread->u.f.fd, dns_check->sbuf, dns_check->slen, 0);
 	if (ret == -1) {
 		if (check_EAGAIN(errno) || check_EINTR(errno)) {
@@ -310,12 +312,12 @@ dns_send(thread_ref_t thread)
 					 checker, thread->u.f.fd, timeout, true);
 			return;
 		}
-		dns_final(thread, 1, "failed to write socket.");
+		dns_final(thread, true, "failed to write socket.");
 		return;
 	}
 
 	if (ret != (ssize_t) dns_check->slen) {
-		dns_final(thread, 1, "failed to write all of the datagram.");
+		dns_final(thread, true, "failed to write all of the datagram.");
 		return;
 	}
 
@@ -324,27 +326,25 @@ dns_send(thread_ref_t thread)
 	return;
 }
 
-static int
+static void
 dns_send_thread(thread_ref_t thread)
 {
 	if (thread->type == THREAD_WRITE_TIMEOUT) {
-		dns_final(thread, 1, "write timeout to socket.");
-		return 0;
+		dns_final(thread, true, "write timeout to socket.");
+		return;
 	}
 
 	dns_send(thread);
-
-	return 0;
 }
 
-static int
+static void
 dns_check_thread(thread_ref_t thread)
 {
 	int status;
 
 	if (thread->type == THREAD_WRITE_TIMEOUT) {
-		dns_final(thread, 1, "write timeout to socket.");
-		return 0;
+		dns_final(thread, true, "write timeout to socket.");
+		return;
 	}
 
 	status = socket_state(thread, dns_check_thread);
@@ -355,13 +355,13 @@ dns_check_thread(thread_ref_t thread)
 	 */
 	switch (status) {
 	case connect_error:
-		dns_final(thread, 1, "connection error.");
+		dns_final(thread, true, "connection error.");
 		break;
 	case connect_timeout:
-		dns_final(thread, 1, "connection timeout.");
+		dns_final(thread, true, "connection timeout.");
 		break;
 	case connect_fail:
-		dns_final(thread, 1, "connection failure.");
+		dns_final(thread, true, "connection failure.");
 		break;
 	case connect_success:
 		dns_make_query(thread);
@@ -372,11 +372,9 @@ dns_check_thread(thread_ref_t thread)
 		thread_del_write(thread);
 		break;
 	}
-
-	return 0;
 }
 
-static int
+static void
 dns_connect_thread(thread_ref_t thread)
 {
 	int fd, status;
@@ -388,7 +386,7 @@ dns_connect_thread(thread_ref_t thread)
 	if (!checker->enabled) {
 		thread_add_timer(thread->master, dns_connect_thread, checker,
 				 checker->delay_loop);
-		return 0;
+		return;
 	}
 
 	if ((fd = socket(co->dst.ss_family, SOCK_DGRAM | SOCK_CLOEXEC | SOCK_NONBLOCK, IPPROTO_UDP)) == -1) {
@@ -396,7 +394,7 @@ dns_connect_thread(thread_ref_t thread)
 				"failed to create socket. Rescheduling.");
 		thread_add_timer(thread->master, dns_connect_thread, checker,
 				 checker->delay_loop);
-		return 0;
+		return;
 	}
 
 #if !HAVE_DECL_SOCK_NONBLOCK
@@ -418,17 +416,18 @@ dns_connect_thread(thread_ref_t thread)
 	if (status == connect_success) {
 		thread_fd = *thread;
 		thread_fd.u.f.fd = fd;
+		thread_fd.sands = timer_add_long(time_now, co->connection_to);
 		dns_make_query(&thread_fd);
 		dns_send(&thread_fd);
 
-		return 0;
+		return;
 	}
 
 	if (status == connect_fail) {
 		close(fd);
-		dns_final(thread, 1, "network unreachable for %s", inet_sockaddrtopair(&co->dst));
+		dns_final(thread, true, "network unreachable for %s", inet_sockaddrtopair(&co->dst));
 
-		return 0;
+		return;
 	}
 
 	/* handle connection status & register check worker thread */
@@ -439,8 +438,6 @@ dns_connect_thread(thread_ref_t thread)
 		thread_add_timer(thread->master, dns_connect_thread, checker,
 				 checker->delay_loop);
 	}
-
-	return 0;
 }
 
 static void
@@ -466,7 +463,7 @@ dns_dump(FILE *fp, const checker_t *checker)
 }
 
 static bool
-dns_check_compare(const checker_t *old_c, const checker_t *new_c)
+dns_check_compare(const checker_t *old_c, checker_t *new_c)
 {
 	const dns_check_t *old = old_c->data;
 	const dns_check_t *new = new_c->data;
@@ -485,10 +482,10 @@ static void
 dns_check_handler(__attribute__((unused)) const vector_t *strvec)
 {
 	checker_t *checker;
+	dns_check_t *dns_check;
 
-	dns_check_t *dns_check = (dns_check_t *) MALLOC(sizeof (dns_check_t));
+	PMALLOC(dns_check);
 	dns_check->type = DNS_DEFAULT_TYPE;
-	dns_check->name = DNS_DEFAULT_NAME;
 	checker = queue_checker(dns_free, dns_dump, dns_connect_thread,
 				dns_check_compare, dns_check, CHECKER_NEW_CO(), true);
 
@@ -515,16 +512,49 @@ static void
 dns_name_handler(const vector_t *strvec)
 {
 	dns_check_t *dns_check = CHECKER_GET();
-	dns_check->name = set_value(strvec);
+	const char *name;
+	bool name_invalid = false;
+	const char *p;
+
+	if (dns_check->name) {
+		report_config_error(CONFIG_GENERAL_ERROR, "DNS_CHECK name already specified - ignoring");
+		return;
+	}
+
+	/* Check name does not have an empty label */
+	name = strvec_slot(strvec, 1);
+	if (name[0] == '.' && name[1] != '\0')
+		name_invalid = true;
+	else {
+		for (p = name; p; p = strchr(p + 1, '.')) {
+			if (p[1] == '.') {
+				name_invalid = true;
+				break;
+			}
+		}
+	}
+
+	if (name_invalid) {
+		report_config_error(CONFIG_GENERAL_ERROR, "DNS_CHECK name '%s' has empty label - ignoring", name);
+		return;
+	}
+
+	dns_check->name = STRDUP(name);
 }
 
 static void
 dns_check_end(void)
 {
+	dns_check_t *dns_check;
+
 	if (!check_conn_opts(CHECKER_GET_CO())) {
 		dequeue_new_checker();
+		return;
 	}
-// Is name needed?
+
+	dns_check = CHECKER_GET();
+	if (!dns_check->name)
+		dns_check->name = STRDUP(DNS_DEFAULT_NAME);
 }
 
 void
