@@ -37,11 +37,13 @@
 
 #include "ipvswrapper.h"
 #include "global_data.h"
-#include "list.h"
 #include "utils.h"
 #include "logger.h"
 #include "libipvs.h"
 #include "main.h"
+#if HAVE_DECL_CLONE_NEWNET
+#include "namespaces.h"
+#endif
 
 static bool no_ipvs = false;
 
@@ -68,15 +70,15 @@ ipvs_cmd_str(int cmd)
 
 /* fetch virtual server group from group name */
 virtual_server_group_t * __attribute__ ((pure))
-ipvs_get_group_by_name(const char *gname, list l)
+ipvs_get_group_by_name(const char *gname, list_head_t *l)
 {
-	element e;
 	virtual_server_group_t *vsg;
 
-	LIST_FOREACH(l, vsg, e) {
+	list_for_each_entry(vsg, l, e_list) {
 		if (!strcmp(vsg->gname, gname))
 			return vsg;
 	}
+
 	return NULL;
 }
 
@@ -104,22 +106,19 @@ ipvs_stop(void)
 	if (no_ipvs)
 		return;
 
+	/* Restore any timeout values we updated */
+	ipvs_set_timeout(NULL);
+
 	ipvs_close();
 }
 
 void
-ipvs_set_timeouts(int tcp_timeout, int tcpfin_timeout, int udp_timeout)
+ipvs_set_timeouts(const ipvs_timeout_t *timeouts)
 {
-	ipvs_timeout_t to;
-
-	if (!tcp_timeout && !tcpfin_timeout && !udp_timeout)
+	if (timeouts && !timeouts->tcp_timeout && !timeouts->tcp_fin_timeout && !timeouts->udp_timeout)
 		return;
 
-	to.tcp_timeout = tcp_timeout;
-	to.tcp_fin_timeout = tcpfin_timeout;
-	to.udp_timeout = udp_timeout;
-
-	if (ipvs_set_timeout(&to))
+	if (ipvs_set_timeout(timeouts))
 		log_message(LOG_INFO, "Failed to set ipvs timeouts");
 }
 
@@ -151,9 +150,11 @@ ipvs_talk(int cmd, ipvs_service_t *srule, ipvs_dest_t *drule, ipvs_daemon_t *dae
 		case IP_VS_SO_SET_EDIT:
 			result = ipvs_update_service(srule);
 			break;
+#ifdef _INCLUDE_UNUSED_CODE_
 		case IP_VS_SO_SET_ZERO:
 			result = ipvs_zero_service(srule);
 			break;
+#endif
 		case IP_VS_SO_SET_ADDDEST:
 			result = ipvs_add_dest(srule, drule);
 			break;
@@ -167,6 +168,8 @@ ipvs_talk(int cmd, ipvs_service_t *srule, ipvs_dest_t *drule, ipvs_daemon_t *dae
 				result = ipvs_add_dest(srule, drule);
 			}
 			break;
+		default:
+			log_message(LOG_INFO, "ipvs_talk() called with unknown command %d", cmd);
 	}
 
 	if (ignore_error)
@@ -183,23 +186,21 @@ ipvs_talk(int cmd, ipvs_service_t *srule, ipvs_dest_t *drule, ipvs_daemon_t *dae
 	return result;
 }
 
-#ifdef _WITH_VRRP_
-/* Note: This function is called in the context of the vrrp child process, not the checker process */
+/* Note: This function may be called in the context of the vrrp child process */
 void
-ipvs_syncd_cmd(int cmd, const struct lvs_syncd_config *config, int state, bool ignore_interface, bool ignore_error)
+ipvs_syncd_cmd(int cmd, const struct lvs_syncd_config *config, int state, bool ignore_error)
 {
 	ipvs_daemon_t daemonrule;
 
 	memset(&daemonrule, 0, sizeof(ipvs_daemon_t));
 
 	/* prepare user rule */
-	daemonrule.state = state;
 	if (config) {
 		daemonrule.syncid = (int)config->syncid;
-		if (!ignore_interface)
-			strcpy_safe(daemonrule.mcast_ifn, config->ifname);
-#ifdef _HAVE_IPVS_SYNCD_ATTRIBUTES_
 		if (cmd == IPVS_STARTDAEMON) {
+			strcpy_safe(daemonrule.mcast_ifn, config->ifname);
+
+#ifdef _HAVE_IPVS_SYNCD_ATTRIBUTES_
 			if (config->sync_maxlen)
 				daemonrule.sync_maxlen = config->sync_maxlen;
 			if (config->mcast_port)
@@ -214,14 +215,24 @@ ipvs_syncd_cmd(int cmd, const struct lvs_syncd_config *config, int state, bool i
 				daemonrule.mcast_af = AF_INET6;
 				memcpy(&daemonrule.mcast_group.in6, &((const struct sockaddr_in6 *)&config->mcast_group)->sin6_addr, sizeof(daemonrule.mcast_group.in6));
 			}
-		}
 #endif
+		}
 	}
 
-	/* Talk to the IPVS channel */
-	ipvs_talk(cmd, NULL, NULL, &daemonrule, ignore_error);
+	if (state & IPVS_MASTER) {
+		daemonrule.state = IP_VS_STATE_MASTER;
+
+		/* Talk to the IPVS channel */
+		ipvs_talk(cmd, NULL, NULL, &daemonrule, ignore_error);
+	}
+
+	if (state & IPVS_BACKUP) {
+		daemonrule.state = IP_VS_STATE_BACKUP;
+
+		/* Talk to the IPVS channel */
+		ipvs_talk(cmd, NULL, NULL, &daemonrule, ignore_error);
+	}
 }
-#endif
 
 void
 ipvs_flush_cmd(void)
@@ -240,6 +251,8 @@ ipvs_group_range_cmd(int cmd, ipvs_service_t *srule, ipvs_dest_t *drule, virtual
 		inet_sockaddrip6(&vsg_entry->addr, &srule->nf_addr.in6);
 	else
 		srule->nf_addr.ip = inet_sockaddrip4(&vsg_entry->addr);
+	srule->af = vsg_entry->addr.ss_family;
+	srule->user.netmask = (srule->af == AF_INET6) ? 128 : ((uint32_t) 0xffffffff);
 
 	/* Process the whole range */
 	for (i = 0; i <= vsg_entry->range; i++) {
@@ -346,16 +359,16 @@ ipvs_group_cmd(int cmd, ipvs_service_t *srule, ipvs_dest_t *drule, virtual_serve
 {
 	virtual_server_group_t *vsg = vs->vsg;
 	virtual_server_group_entry_t *vsg_entry;
-	element e;
 
 	/* return if jointure fails */
 	if (!vsg)
 		return 0;
 
 	/* visit addr_range list */
-	LIST_FOREACH(vsg->addr_range, vsg_entry, e) {
+	list_for_each_entry(vsg_entry, &vsg->addr_range, e_list) {
 		if (cmd == IP_VS_SO_SET_ADD && reload && vsg_entry->reloaded)
 			continue;
+
 		if (ipvs_change_needed(cmd, vsg_entry, vs, rs)) {
 			srule->user.port = inet_sockaddrport(&vsg_entry->addr);
 			if (rs) {
@@ -382,17 +395,24 @@ ipvs_group_cmd(int cmd, ipvs_service_t *srule, ipvs_dest_t *drule, virtual_serve
 		else
 			drule->user.port = inet_sockaddrport(&rs->addr);
 	}
-	LIST_FOREACH(vsg->vfwmark, vsg_entry, e) {
+
+	list_for_each_entry(vsg_entry, &vsg->vfwmark, e_list) {
 		if (cmd == IP_VS_SO_SET_ADD && reload && vsg_entry->reloaded)
 			continue;
 
 		srule->user.fwmark = vsg_entry->vfwmark;
 
+		if (vsg_entry->fwm_family != AF_UNSPEC) {
+			srule->af = vsg_entry->fwm_family;
+			srule->user.netmask = (srule->af == AF_INET6) ? 128 : ((uint32_t) 0xffffffff);
+		}
+
 		/* Talk to the IPVS channel */
 		if (ipvs_change_needed(cmd, vsg_entry, vs, rs)) {
 			if (ipvs_talk(cmd, srule, drule, NULL, false))
 				return -1;
 		}
+
 		ipvs_set_vsge_alive_state(cmd, vsg_entry, vs);
 	}
 
@@ -413,17 +433,17 @@ ipvs_set_srule(int cmd, ipvs_service_t *srule, virtual_server_t *vs)
 	srule->user.protocol = vs->service_type;
 
 	if (vs->persistence_timeout &&
-	    (cmd == IP_VS_SO_SET_ADD || cmd == IP_VS_SO_SET_DEL)) {
+	    (cmd == IP_VS_SO_SET_ADD || cmd == IP_VS_SO_SET_DEL || cmd == IP_VS_SO_SET_EDIT)) {
 		srule->user.timeout = vs->persistence_timeout;
 		srule->user.flags |= IP_VS_SVC_F_PERSISTENT;
 
 		if (vs->persistence_granularity != 0xffffffff)
 			srule->user.netmask = vs->persistence_granularity;
-	}
 
 #ifdef _HAVE_PE_NAME_
-	strcpy(srule->pe_name, vs->pe_name);
+		strcpy(srule->pe_name, vs->pe_name);
 #endif
+	}
 }
 
 /* Fill IPVS rule with rs infos */
@@ -514,7 +534,6 @@ void
 ipvs_group_sync_entry(virtual_server_t *vs, virtual_server_group_entry_t *vsge)
 {
 	real_server_t *rs;
-	element e;
 	ipvs_service_t srule;
 	ipvs_dest_t drule;
 
@@ -525,7 +544,7 @@ ipvs_group_sync_entry(virtual_server_t *vs, virtual_server_group_entry_t *vsge)
 		srule.user.port = inet_sockaddrport(&vsge->addr);
 
 	/* Process realserver queue */
-	LIST_FOREACH(vs->rs, rs, e) {
+	list_for_each_entry(rs, &vs->rs, e_list) {
 // ??? What if !quorum_state_up?
 		if (rs->reloaded && (rs->alive || (rs->inhibit && rs->set))) {
 			/* Prepare the IPVS drule */
@@ -548,7 +567,6 @@ void
 ipvs_group_remove_entry(virtual_server_t *vs, virtual_server_group_entry_t *vsge)
 {
 	real_server_t *rs;
-	element e;
 	ipvs_service_t srule;
 	ipvs_dest_t drule;
 
@@ -560,7 +578,7 @@ ipvs_group_remove_entry(virtual_server_t *vs, virtual_server_group_entry_t *vsge
 		srule.user.port = inet_sockaddrport(&vsge->addr);
 
 	/* Process realserver queue */
-	LIST_FOREACH(vs->rs, rs, e) {
+	list_for_each_entry(rs, &vs->rs, e_list) {
 		if (rs->alive) {
 			/* Setting IPVS drule */
 			ipvs_set_drule(IP_VS_SO_SET_DELDEST, &drule, rs);
@@ -609,9 +627,8 @@ vsd_equal(real_server_t *rs, struct ip_vs_dest_entry_app *entry)
 static void
 ipvs_update_vs_stats(virtual_server_t *vs, uint32_t fwmark, union nf_inet_addr *nfaddr, uint16_t port)
 {
-	element e;
 	struct ip_vs_get_dests_app *dests = NULL;
-	real_server_t *rs;
+	real_server_t *rs, *rs_match;
 	unsigned int i;
 	ipvs_service_entry_t *serv;
 
@@ -638,18 +655,20 @@ ipvs_update_vs_stats(virtual_server_t *vs, uint32_t fwmark, union nf_inet_addr *
 
 	for (i = 0; i < dests->user.num_dests; i++) {
 		rs = NULL;
+		rs_match = NULL;
 
 		/* Is it the sorry server? */
 		if (vs->s_svr && vsd_equal(vs->s_svr, &dests->user.entrytable[i]))
 			rs = vs->s_svr;
 		else {
 			/* Search for a match in the list of real servers */
-			for (e = LIST_HEAD(vs->rs); e; ELEMENT_NEXT(e)) {
-				rs = ELEMENT_DATA(e);
-				if (vsd_equal(rs, &dests->user.entrytable[i]))
+			list_for_each_entry(rs, &vs->rs, e_list) {
+				if (vsd_equal(rs, &dests->user.entrytable[i])) {
+					rs_match = rs;
 					break;
+				}
 			}
-			if (!e)
+			if (!rs_match)
 				rs = NULL;
 		}
 
@@ -678,7 +697,6 @@ ipvs_update_vs_stats(virtual_server_t *vs, uint32_t fwmark, union nf_inet_addr *
 void
 ipvs_update_stats(virtual_server_t *vs)
 {
-	element e, ge;
 	virtual_server_group_entry_t *vsg_entry;
 	uint32_t addr_ip;
 	uint16_t port;
@@ -698,20 +716,17 @@ ipvs_update_stats(virtual_server_t *vs)
 		vs->s_svr->activeconns =
 			vs->s_svr->inactconns = vs->s_svr->persistconns = 0;
 	}
-	for (e = LIST_HEAD(vs->rs); e; ELEMENT_NEXT(e)) {
-		rs = ELEMENT_DATA(e);
+	list_for_each_entry(rs, &vs->rs, e_list) {
 		memset(&rs->stats, 0, sizeof(rs->stats));
 		rs->activeconns = rs->inactconns = rs->persistconns = 0;
 	}
 
 	/* Update the stats */
 	if (vs->vsg) {
-		for (ge = LIST_HEAD(vs->vsg->vfwmark); ge; ELEMENT_NEXT(ge)) {
-			vsg_entry = ELEMENT_DATA(ge);
+		list_for_each_entry(vsg_entry, &vs->vsg->vfwmark, e_list)
 			ipvs_update_vs_stats(vs, vsg_entry->vfwmark, &nfaddr, 0);
-		}
-		for (ge = LIST_HEAD(vs->vsg->addr_range); ge; ELEMENT_NEXT(ge)) {
-			vsg_entry = ELEMENT_DATA(ge);
+
+		list_for_each_entry(vsg_entry, &vs->vsg->addr_range, e_list) {
 			addr_ip = (vsg_entry->addr.ss_family == AF_INET6) ?
 				    ntohs(((struct sockaddr_in6 *)&vsg_entry->addr)->sin6_addr.s6_addr16[7]) :
 				    ntohl(((struct sockaddr_in *)&vsg_entry->addr)->sin_addr.s_addr);
@@ -741,6 +756,21 @@ ipvs_update_stats(virtual_server_t *vs)
 }
 #endif /* _WITH_SNMP_CHECKER_ */
 
+bool
+ipvs_syncd_changed(const struct lvs_syncd_config *old, const struct lvs_syncd_config *new)
+{
+	return (old->syncid != new->syncid ||
+		strcmp(old->ifname, new->ifname)
+#ifdef _HAVE_IPVS_SYNCD_ATTRIBUTES_
+						 ||
+		old->sync_maxlen != new->sync_maxlen ||
+		old->mcast_port != new->mcast_port ||
+		old->mcast_ttl != new->mcast_ttl ||
+		!sockstorage_equal(&old->mcast_group, &new->mcast_group)
+#endif
+						);
+}
+
 #ifdef _WITH_VRRP_
 /*
  * Common IPVS functions
@@ -749,15 +779,15 @@ ipvs_update_stats(virtual_server_t *vs)
 void
 ipvs_syncd_master(const struct lvs_syncd_config *config)
 {
-	ipvs_syncd_cmd(IPVS_STOPDAEMON, config, IPVS_BACKUP, false, false);
-	ipvs_syncd_cmd(IPVS_STARTDAEMON, config, IPVS_MASTER, false, false);
+	ipvs_syncd_cmd(IPVS_STOPDAEMON, config, IPVS_BACKUP, false);
+	ipvs_syncd_cmd(IPVS_STARTDAEMON, config, IPVS_MASTER, false);
 }
 
 /* Note: This function is called in the context of the vrrp child process, not the checker process */
 void
 ipvs_syncd_backup(const struct lvs_syncd_config *config)
 {
-	ipvs_syncd_cmd(IPVS_STOPDAEMON, config, IPVS_MASTER, false, false);
-	ipvs_syncd_cmd(IPVS_STARTDAEMON, config, IPVS_BACKUP, false, false);
+	ipvs_syncd_cmd(IPVS_STOPDAEMON, config, IPVS_MASTER, false);
+	ipvs_syncd_cmd(IPVS_STARTDAEMON, config, IPVS_BACKUP, false);
 }
 #endif
