@@ -24,9 +24,10 @@
 
 #include <unistd.h>
 #include <pwd.h>
+#include <sched.h>
 
 #include "global_data.h"
-#include "list.h"
+#include "list_head.h"
 #include "logger.h"
 #include "parser.h"
 #include "utils.h"
@@ -105,12 +106,7 @@ set_vrrp_defaults(data_t * data)
 	data->vrrp_higher_prio_send_advert = false;
 	data->vrrp_version = VRRP_VERSION_2;
 #ifdef _HAVE_LIBIPSET_
-	data->using_ipsets = true;
-	strcpy(data->vrrp_ipset_address, DEFAULT_IPSET_NAME);
-	strcpy(data->vrrp_ipset_address6, DEFAULT_IPSET_NAME "6");
-	strcpy(data->vrrp_ipset_address_iface6, DEFAULT_IPSET_NAME "if6");
-	strcpy(data->vrrp_ipset_igmp, DEFAULT_IPSET_NAME "_igmp");
-	strcpy(data->vrrp_ipset_mld, DEFAULT_IPSET_NAME "_mld");
+	data->using_ipsets = PARAMETER_UNSET;
 #endif
 	data->vrrp_check_unicast_src = false;
 	data->vrrp_skip_check_adv_addr = false;
@@ -123,21 +119,34 @@ set_vrrp_defaults(data_t * data)
 
 /* email facility functions */
 static void
-free_email(void *data)
+free_email_list(list_head_t *l)
 {
-	FREE(data);
+	email_t *email, *email_tmp;
+
+	list_for_each_entry_safe(email, email_tmp, l, e_list) {
+		FREE(email->addr);
+		FREE(email);
+	}
 }
 static void
-dump_email(FILE *fp, const void *data)
+dump_email_list(FILE *fp, const list_head_t *l)
 {
-	const char *addr = data;
-	conf_write(fp, "   %s", addr);
+	email_t *email;
+
+	list_for_each_entry(email, l, e_list)
+		conf_write(fp, "   %s", email->addr);
 }
 
 void
 alloc_email(const char *addr)
 {
-	list_add(global_data->email, STRDUP(addr));
+	email_t *email;
+
+	PMALLOC(email);
+	INIT_LIST_HEAD(&email->e_list);
+	email->addr = STRDUP(addr);
+
+	list_add_tail(&email->e_list, &global_data->email);
 }
 
 /* data facility functions */
@@ -149,8 +158,8 @@ alloc_global_data(void)
 	if (global_data)
 		return global_data;
 
-	new = MALLOC(sizeof(data_t));
-	new->email = alloc_list(free_email, dump_email);
+	PMALLOC(new);
+	INIT_LIST_HEAD(&new->email);
 	new->smtp_alert = -1;
 #ifdef _WITH_VRRP_
 	new->smtp_alert_vrrp = -1;
@@ -164,6 +173,8 @@ alloc_global_data(void)
 	set_vrrp_defaults(new);
 #endif
 	new->notify_fifo.fd = -1;
+	new->max_auto_priority = 0;
+	new->min_auto_priority_delay = 1000000;	/* 1 second */
 #ifdef _WITH_VRRP_
 	new->vrrp_notify_fifo.fd = -1;
 #if HAVE_DECL_RLIMIT_RTTIME == 1
@@ -229,10 +240,16 @@ init_global_data(data_t * data, data_t *prev_global_data, bool copy_unchangeable
 		prev_global_data->local_name = NULL;
 
 		if (copy_unchangeable_config) {
+#if HAVE_DECL_CLONE_NEWNET
 			FREE_CONST_PTR(data->network_namespace);
 			data->network_namespace = prev_global_data->network_namespace;
 			prev_global_data->network_namespace = NULL;
 
+			FREE_CONST_PTR(data->network_namespace_ipvs);
+			data->network_namespace_ipvs = prev_global_data->network_namespace_ipvs;
+			prev_global_data->network_namespace_ipvs = NULL;
+#endif
+
 			FREE_CONST_PTR(data->instance_name);
 			data->instance_name = prev_global_data->instance_name;
 			prev_global_data->instance_name = NULL;
@@ -320,9 +337,10 @@ free_global_data(data_t * data)
 	if (!data)
 		return;
 
-	free_list(&data->email);
+	free_email_list(&data->email);
 #if HAVE_DECL_CLONE_NEWNET
 	FREE_CONST_PTR(data->network_namespace);
+	FREE_CONST_PTR(data->network_namespace_ipvs);
 #endif
 	FREE_CONST_PTR(data->instance_name);
 	FREE_CONST_PTR(data->process_name);
@@ -342,16 +360,34 @@ free_global_data(data_t * data)
 #ifdef _WITH_SNMP_
 	FREE_CONST_PTR(data->snmp_socket);
 #endif
+	free_notify_script(&data->startup_script);
+	free_notify_script(&data->shutdown_script);
 #if defined _WITH_LVS_ && defined _WITH_VRRP_
 	FREE_CONST_PTR(data->lvs_syncd.ifname);
 	FREE_CONST_PTR(data->lvs_syncd.vrrp_name);
 #endif
 	FREE_CONST_PTR(data->notify_fifo.name);
+#ifndef _ONE_PROCESS_DEBUG_
+	FREE_CONST_PTR(data->reload_time_file);
+#endif
 	free_notify_script(&data->notify_fifo.script);
 #ifdef _WITH_VRRP_
 	FREE_CONST_PTR(data->default_ifname);
 	FREE_CONST_PTR(data->vrrp_notify_fifo.name);
 	free_notify_script(&data->vrrp_notify_fifo.script);
+#ifdef _WITH_IPTABLES_
+	FREE_CONST_PTR(data->vrrp_iptables_inchain);
+	FREE_CONST_PTR(data->vrrp_iptables_outchain);
+#ifdef _HAVE_LIBIPSET_
+	FREE_CONST_PTR(data->vrrp_ipset_address);
+	FREE_CONST_PTR(data->vrrp_ipset_address6);
+	FREE_CONST_PTR(data->vrrp_ipset_address_iface6);
+#ifdef HAVE_IPSET_ATTR_IFACE
+	FREE_CONST_PTR(data->vrrp_ipset_igmp);
+	FREE_CONST_PTR(data->vrrp_ipset_mld);
+#endif
+#endif
+#endif
 #ifdef _WITH_NFTABLES_
 	FREE_CONST_PTR(data->vrrp_nf_table_name);
 #endif
@@ -373,6 +409,11 @@ dump_global_data(FILE *fp, data_t * data)
 #ifdef _WITH_VRRP_
 	char buf[64];
 #endif
+#ifndef _ONE_PROCESS_DEBUG_
+	char date_time_str[20];
+	struct tm tm;
+#endif
+	unsigned val;
 
 	if (!data)
 		return;
@@ -381,6 +422,7 @@ dump_global_data(FILE *fp, data_t * data)
 
 #if HAVE_DECL_CLONE_NEWNET
 	conf_write(fp, " Network namespace = %s", data->network_namespace ? data->network_namespace : "(default)");
+	conf_write(fp, " Network namespace ipvs = %s", data->network_namespace_ipvs ? data->network_namespace_ipvs[0] ? data->network_namespace_ipvs : "(default)" : "(main namespace)");
 #endif
 	if (data->instance_name)
 		conf_write(fp, " Instance name = %s", data->instance_name);
@@ -413,7 +455,7 @@ dump_global_data(FILE *fp, data_t * data)
 		conf_write(fp, " Email notification from = %s"
 				    , data->email_from);
 		conf_write(fp, " Email notification to:");
-		dump_list(fp, data->email);
+		dump_email_list(fp, &data->email);
 	}
 	conf_write(fp, " Default smtp_alert = %s",
 			data->smtp_alert == -1 ? "unset" : data->smtp_alert ? "on" : "off");
@@ -426,6 +468,29 @@ dump_global_data(FILE *fp, data_t * data)
 			data->smtp_alert_checker == -1 ? "unset" : data->smtp_alert_checker ? "on" : "off");
 	conf_write(fp, " Checkers log all failures = %s", data->checker_log_all_failures ? "true" : "false");
 #endif
+#ifndef _ONE_PROCESS_DEBUG_
+	if (data->reload_time_file) {
+		conf_write(fp, " Reload time file = %s%s", data->reload_time_file, data->reload_repeat ? " (repeat)" : "");
+		if (data->reload_time) {
+			localtime_r(&data->reload_time, &tm);
+			strftime(date_time_str, sizeof(date_time_str), "%Y-%m-%d %H:%M:%S", &tm);
+			conf_write(fp, " Reload scheduled for %s%s", date_time_str, global_data->reload_date_specified ? " (date specified" : "");
+		} else
+			conf_write(fp, " No reload scheduled");
+	}
+#endif
+	if (data->startup_script)
+		conf_write(fp, " Startup script = %s, uid:gid %u:%u, timeout %u",
+			    cmd_str(data->startup_script),
+			    data->startup_script->uid,
+			    data->startup_script->gid,
+			    data->startup_script_timeout);
+	if (data->shutdown_script)
+		conf_write(fp, " Shutdown script = %s, uid:gid %u:%u timeout %u",
+			    cmd_str(data->shutdown_script),
+			    data->shutdown_script->uid,
+			    data->shutdown_script->gid,
+			    data->shutdown_script_timeout);
 #ifdef _WITH_VRRP_
 	conf_write(fp, " Dynamic interfaces = %s", data->dynamic_interfaces ? "true" : "false");
 	if (data->dynamic_interfaces)
@@ -434,23 +499,26 @@ dump_global_data(FILE *fp, data_t * data)
 		conf_write(fp, " Send emails for fault transitions = off");
 #endif
 #ifdef _WITH_LVS_
-	if (data->lvs_tcp_timeout)
-		conf_write(fp, " LVS TCP timeout = %d", data->lvs_tcp_timeout);
-	if (data->lvs_tcpfin_timeout)
-		conf_write(fp, " LVS TCP FIN timeout = %d", data->lvs_tcpfin_timeout);
-	if (data->lvs_udp_timeout)
-		conf_write(fp, " LVS TCP timeout = %d", data->lvs_udp_timeout);
+	if (data->lvs_timeouts.tcp_timeout)
+		conf_write(fp, " LVS TCP timeout = %d", data->lvs_timeouts.tcp_timeout);
+	if (data->lvs_timeouts.tcp_fin_timeout)
+		conf_write(fp, " LVS TCP FIN timeout = %d", data->lvs_timeouts.tcp_fin_timeout);
+	if (data->lvs_timeouts.udp_timeout)
+		conf_write(fp, " LVS TCP timeout = %d", data->lvs_timeouts.udp_timeout);
 #ifdef _WITH_VRRP_
 #ifndef _ONE_PROCESS_DEBUG_
 	if (prog_type == PROG_TYPE_VRRP)
 #endif
 		conf_write(fp, " Default interface = %s", data->default_ifp ? data->default_ifp->ifname : DFLT_INT);
-	if (data->lvs_syncd.vrrp) {
-		conf_write(fp, " LVS syncd vrrp instance = %s"
-				    , data->lvs_syncd.vrrp->iname);
-		if (data->lvs_syncd.ifname)
-			conf_write(fp, " LVS syncd interface = %s"
-				    , data->lvs_syncd.ifname);
+	if (data->lvs_syncd.ifname) {
+		if (data->lvs_syncd.vrrp)
+			conf_write(fp, " LVS syncd vrrp instance = %s"
+				     , data->lvs_syncd.vrrp->iname);
+		else if (data->lvs_syncd.vrrp_name)
+			conf_write(fp, " LVS syncd vrrp name = %s"
+				     , data->lvs_syncd.vrrp_name);
+		conf_write(fp, " LVS syncd interface = %s"
+			     , data->lvs_syncd.ifname);
 		conf_write(fp, " LVS syncd syncid = %u"
 				    , data->lvs_syncd.syncid);
 #ifdef _HAVE_IPVS_SYNCD_ATTRIBUTES_
@@ -520,23 +588,25 @@ dump_global_data(FILE *fp, data_t * data)
 	conf_write(fp, " Gratuitous NA interval = %f", data->vrrp_gna_interval / TIMER_HZ_DOUBLE);
 	conf_write(fp, " VRRP default protocol version = %d", data->vrrp_version);
 #ifdef _WITH_IPTABLES_
-	if (data->vrrp_iptables_inchain[0]) {
+	if (data->vrrp_iptables_inchain) {
 		conf_write(fp," Iptables input chain = %s", data->vrrp_iptables_inchain);
-		if (data->vrrp_iptables_outchain[0])
+		if (data->vrrp_iptables_outchain)
 			conf_write(fp," Iptables output chain = %s", data->vrrp_iptables_outchain);
 #ifdef _HAVE_LIBIPSET_
 		conf_write(fp, " Using ipsets = %s", data->using_ipsets ? "true" : "false");
 		if (data->using_ipsets) {
-			if (data->vrrp_ipset_address[0])
+			if (data->vrrp_ipset_address)
 				conf_write(fp," ipset IPv4 address set = %s", data->vrrp_ipset_address);
-			if (data->vrrp_ipset_address6[0])
+			if (data->vrrp_ipset_address6)
 				conf_write(fp," ipset IPv6 address set = %s", data->vrrp_ipset_address6);
-			if (data->vrrp_ipset_address_iface6[0])
+			if (data->vrrp_ipset_address_iface6)
 				conf_write(fp," ipset IPv6 address,iface set = %s", data->vrrp_ipset_address_iface6);
-			if (data->vrrp_ipset_igmp[0])
+#ifdef HAVE_IPSET_ATTR_IFACE
+			if (data->vrrp_ipset_igmp)
 				conf_write(fp," ipset IGMP set = %s", data->vrrp_ipset_igmp);
-			if (data->vrrp_ipset_mld[0])
+			if (data->vrrp_ipset_mld)
 				conf_write(fp," ipset MLD set = %s", data->vrrp_ipset_mld);
+#endif
 		}
 #endif
 	}
@@ -547,9 +617,6 @@ dump_global_data(FILE *fp, data_t * data)
 		conf_write(fp," nftables base chain priority = %d", data->vrrp_nf_chain_priority);
 		conf_write(fp," nftables with%s counters", data->vrrp_nf_counters ? "" : "out");
 		conf_write(fp," nftables %sforce use ifindex for link local IPv6", data->vrrp_nf_ifindex ? "" : "don't ");
-		if (data->nft_version)
-			conf_write(fp," nft version %u.%u.%u", data->nft_version >> 16,
-					(data->nft_version >> 8) & 0xff, data->nft_version & 0xff);
 		conf_write(fp," libnftnl version %u.%u.%u", LIBNFTNL_VERSION >> 16,
 			       (LIBNFTNL_VERSION >> 8) & 0xff, LIBNFTNL_VERSION & 0xff);
 	}
@@ -558,6 +625,11 @@ dump_global_data(FILE *fp, data_t * data)
 	conf_write(fp, " VRRP check unicast_src = %s", data->vrrp_check_unicast_src ? "true" : "false");
 	conf_write(fp, " VRRP skip check advert addresses = %s", data->vrrp_skip_check_adv_addr ? "true" : "false");
 	conf_write(fp, " VRRP strict mode = %s", data->vrrp_strict ? "true" : "false");
+	if (data->max_auto_priority == -1)
+		conf_write(fp, " Max auto priority = Disabled");
+	else
+		conf_write(fp, " Max auto priority = %d", data->max_auto_priority);
+	conf_write(fp, " Min auto priority delay = %ld usecs", data->min_auto_priority_delay);
 	conf_write(fp, " VRRP process priority = %d", data->vrrp_process_priority);
 	conf_write(fp, " VRRP don't swap = %s", data->vrrp_no_swap ? "true" : "false");
 	conf_write(fp, " VRRP realtime priority = %u", data->vrrp_realtime_priority);
@@ -649,5 +721,11 @@ dump_global_data(FILE *fp, data_t * data)
 		conf_write(fp, " vrrp_startup_delay = %g", global_data->vrrp_startup_delay / TIMER_HZ_DOUBLE);
 	if (global_data->log_unknown_vrids)
 		conf_write(fp, " log_unknown_vrids");
+#endif
+	if ((val = get_cur_priority()))
+		conf_write(fp, " current realtime priority = %u", val);
+#if HAVE_DECL_RLIMIT_RTTIME
+	if ((val = get_cur_rlimit_rttime()))
+		conf_write(fp, " current realtime time limit = %u", val);
 #endif
 }
