@@ -62,10 +62,6 @@
 #define TASK_COMM_LEN	16
 #endif
 
-#ifdef _WITH_LVS_
-#define LVS_MAX_TIMEOUT		(86400*31)	/* 31 days */
-#endif
-
 /* data handlers */
 /* Global def handlers */
 #ifdef _WITH_LINKBEAT_
@@ -263,6 +259,115 @@ smtp_alert_handler(const vector_t *strvec)
 
 	global_data->smtp_alert = res;
 }
+
+static void
+startup_shutdown_script(const vector_t *strvec, notify_script_t **script, bool startup)
+{
+	const char *type = startup ? "startup" : "shutdown";
+
+	if (*script) {
+		report_config_error(CONFIG_GENERAL_ERROR, "%s script already specified", type);
+		return;
+	}
+
+	if (vector_size(strvec) < 2) {
+		report_config_error(CONFIG_GENERAL_ERROR, "%s script missing", type);
+		return;
+	}
+
+	if (!(*script = notify_script_init(0, type))) {
+		report_config_error(CONFIG_GENERAL_ERROR, "Invalid %s script", type);
+		return;
+	}
+
+	if (startup) {
+		if (!global_data->startup_script_timeout)
+			global_data->startup_script_timeout = 10;
+	} else {
+		if (!global_data->shutdown_script_timeout)
+			global_data->shutdown_script_timeout = 10;
+	}
+}
+
+static void
+startup_shutdown_script_timeout_handler(const vector_t *strvec, bool startup)
+{
+	const char *type = startup ? "startup" : "shutdown";
+	unsigned delay;
+
+	if (vector_size(strvec) < 2) {
+		report_config_error(CONFIG_GENERAL_ERROR, "%s_script_timeout requires value", type);
+		return;
+	}
+	if (!read_unsigned_strvec(strvec, 1, &delay, 1, 1000, true)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "%s_script_timeout '%s' must be in [1, 1000] - ignoring", type, strvec_slot(strvec, 1));
+		return;
+	}
+
+	if (startup)
+		global_data->startup_script_timeout = delay;
+	else
+		global_data->shutdown_script_timeout = delay;
+}
+
+static void
+startup_script_handler(const vector_t *strvec)
+{
+	startup_shutdown_script(strvec, &global_data->startup_script, true);
+}
+
+static void
+startup_script_timeout_handler(const vector_t *strvec)
+{
+	startup_shutdown_script_timeout_handler(strvec, true);
+}
+
+static void
+shutdown_script_handler(const vector_t *strvec)
+{
+	startup_shutdown_script(strvec, &global_data->shutdown_script, false);
+}
+
+static void
+shutdown_script_timeout_handler(const vector_t *strvec)
+{
+	startup_shutdown_script_timeout_handler(strvec, false);
+}
+
+static void
+max_auto_priority_handler(const vector_t *strvec)
+{
+	int priority;
+	int max_priority = sched_get_priority_max(SCHED_RR);
+
+	if (vector_size(strvec) < 2) {
+		global_data->max_auto_priority = max_priority;
+		return;
+	}
+
+	if (!read_int_strvec(strvec, 1, &priority, -1, max_priority, true)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "max_auto_priority '%s' must be in [0, %d] (or -1 to disable) - ignoring", strvec_slot(strvec, 1), max_priority);
+		return;
+	}
+
+	global_data->max_auto_priority = priority;
+}
+static void
+min_auto_priority_delay_handler(const vector_t *strvec)
+{
+	int delay;
+
+	if (vector_size(strvec) < 2) {
+		report_config_error(CONFIG_GENERAL_ERROR, "min_auto_priority_delay requires delay time");
+		return;
+	}
+	if (!read_int_strvec(strvec, 1, &delay, 1, 10000000, true)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "min_auto_priority_delay '%s' must be in [1, 10000000] - ignoring", strvec_slot(strvec, 1));
+		return;
+	}
+
+	global_data->min_auto_priority_delay = delay;
+}
 #ifdef _WITH_VRRP_
 static void
 smtp_alert_vrrp_handler(const vector_t *strvec)
@@ -322,16 +427,6 @@ default_interface_handler(const vector_t *strvec)
 	}
 	FREE_CONST_PTR(global_data->default_ifname);
 	global_data->default_ifname = set_value(strvec);
-
-	/* On a reload, the VRRP process needs the default_ifp */
-#ifndef _ONE_PROCESS_DEBUG_
-	if (prog_type == PROG_TYPE_VRRP)
-#endif
-	{
-		global_data->default_ifp = if_get_by_ifname(global_data->default_ifname, IF_CREATE_IF_DYNAMIC);
-		if (!global_data->default_ifp)
-			report_config_error(CONFIG_GENERAL_ERROR, "WARNING - default interface %s doesn't exist", global_data->default_ifname);
-	}
 }
 #endif
 #ifdef _WITH_LVS_
@@ -355,7 +450,7 @@ lvs_timeouts(const vector_t *strvec)
 			if (!read_unsigned_strvec(strvec, i + 1, &val, 0, LVS_MAX_TIMEOUT, false))
 				report_config_error(CONFIG_GENERAL_ERROR, "Invalid lvs_timeout tcp (%s) - ignoring", strvec_slot(strvec, i+1));
 			else
-				global_data->lvs_tcp_timeout = val;
+				global_data->lvs_timeouts.tcp_timeout = val;
 			i++;	/* skip over value */
 			continue;
 		}
@@ -367,7 +462,7 @@ lvs_timeouts(const vector_t *strvec)
 			if (!read_unsigned_strvec(strvec, i + 1, &val, 0, LVS_MAX_TIMEOUT, false))
 				report_config_error(CONFIG_GENERAL_ERROR, "Invalid lvs_timeout tcpfin (%s) - ignoring", strvec_slot(strvec, i+1));
 			else
-				global_data->lvs_tcpfin_timeout = val;
+				global_data->lvs_timeouts.tcp_fin_timeout = val;
 			i++;	/* skip over value */
 			continue;
 		}
@@ -379,7 +474,7 @@ lvs_timeouts(const vector_t *strvec)
 			if (!read_unsigned_strvec(strvec, i + 1, &val, 0, LVS_MAX_TIMEOUT, false))
 				report_config_error(CONFIG_GENERAL_ERROR, "Invalid lvs_timeout udp (%s) - ignoring", strvec_slot(strvec, i+1));
 			else
-				global_data->lvs_udp_timeout = val;
+				global_data->lvs_timeouts.udp_timeout = val;
 			i++;	/* skip over value */
 			continue;
 		}
@@ -394,12 +489,12 @@ lvs_syncd_handler(const vector_t *strvec)
 	size_t i;
 
 	if (global_data->lvs_syncd.ifname) {
-		report_config_error(CONFIG_GENERAL_ERROR, "lvs_sync_daemon has already been specified as %s %s - ignoring", global_data->lvs_syncd.ifname, global_data->lvs_syncd.vrrp_name);
+		report_config_error(CONFIG_GENERAL_ERROR, "lvs_sync_daemon has already been specified as %s %s - ignoring", global_data->lvs_syncd.ifname, global_data->lvs_syncd.vrrp_name ? global_data->lvs_syncd.vrrp_name : "");
 		return;
 	}
 
-	if (vector_size(strvec) < 3) {
-		report_config_error(CONFIG_GENERAL_ERROR, "lvs_sync_daemon requires interface, VRRP instance");
+	if (vector_size(strvec) < 2) {
+		report_config_error(CONFIG_GENERAL_ERROR, "lvs_sync_daemon requires interface");
 		return;
 	}
 
@@ -408,29 +503,20 @@ lvs_syncd_handler(const vector_t *strvec)
 		return;
 	}
 
-	if (strlen(strvec_slot(strvec, 2)) >= IP_VS_IFNAME_MAXLEN) {
-		report_config_error(CONFIG_GENERAL_ERROR, "lvs_sync_daemon vrrp interface name '%s' too long - ignoring", strvec_slot(strvec, 2));
-		return;
-	}
-
 	global_data->lvs_syncd.ifname = set_value(strvec);
 
-	if (!(global_data->lvs_syncd.vrrp_name = STRDUP(strvec_slot(strvec, 2))))
-		return;
+	for (i = 2; i < vector_size(strvec); i++) {
+#ifdef _WITH_VRRP_
+		if (!strcmp(strvec_slot(strvec, i), "inst")) {
+			if (global_data->lvs_syncd.vrrp_name)
+				report_config_error(CONFIG_GENERAL_ERROR, "lvs_sync_daemon vrrp instance has already been specified as %s - ignoring", global_data->lvs_syncd.vrrp_name);
+			else
+				global_data->lvs_syncd.vrrp_name = STRDUP(strvec_slot(strvec, i + 1));
 
-	/* This is maintained for backwards compatibility, prior to adding "id" option */
-	if (vector_size(strvec) >= 4 && isdigit(strvec_slot(strvec, 3)[0])) {
-		report_config_error(CONFIG_GENERAL_ERROR, "Please use keyword \"id\" before lvs_sync_daemon syncid value");
-		if (!read_unsigned_strvec(strvec, 3, &val, 0, 255, false))
-			report_config_error(CONFIG_GENERAL_ERROR, "Invalid syncid (%s) - defaulting to vrid", strvec_slot(strvec, 3));
-		else
-			global_data->lvs_syncd.syncid = val;
-		i = 4;
-	}
-	else
-		i = 3;
-
-	for ( ; i < vector_size(strvec); i++) {
+			i++;	/* skip over value */
+			continue;
+		}
+#endif
 		if (!strcmp(strvec_slot(strvec, i), "id")) {
 			if (i == vector_size(strvec) - 1) {
 				report_config_error(CONFIG_GENERAL_ERROR, "No value specified for lvs_sync_daemon id, defaulting to vrid");
@@ -449,7 +535,7 @@ lvs_syncd_handler(const vector_t *strvec)
 				report_config_error(CONFIG_GENERAL_ERROR, "No value specified for lvs_sync_daemon maxlen - ignoring");
 				continue;
 			}
-			if (!read_unsigned_strvec(strvec, i + 1, &val, 0, 65535 - 20 - 8, false))
+			if (!read_unsigned_strvec(strvec, i + 1, &val, 1, 65535 - 20 - 8, false))
 				report_config_error(CONFIG_GENERAL_ERROR, "Invalid lvs_sync_daemon maxlen (%s) - ignoring", strvec_slot(strvec, i+1));
 			else
 				global_data->lvs_syncd.sync_maxlen = (uint16_t)val;
@@ -461,7 +547,7 @@ lvs_syncd_handler(const vector_t *strvec)
 				report_config_error(CONFIG_GENERAL_ERROR, "No value specified for lvs_sync_daemon port - ignoring");
 				continue;
 			}
-			if (!read_unsigned_strvec(strvec, i + 1, &val, 0, 65535, false))
+			if (!read_unsigned_strvec(strvec, i + 1, &val, 1, 65535, false))
 				report_config_error(CONFIG_GENERAL_ERROR, "Invalid lvs_sync_daemon port (%s) - ignoring", strvec_slot(strvec, i+1));
 			else
 				global_data->lvs_syncd.mcast_port = (uint16_t)val;
@@ -473,7 +559,7 @@ lvs_syncd_handler(const vector_t *strvec)
 				report_config_error(CONFIG_GENERAL_ERROR, "No value specified for lvs_sync_daemon ttl - ignoring");
 				continue;
 			}
-			if (!read_unsigned_strvec(strvec, i + 1, &val, 0, 255, false))
+			if (!read_unsigned_strvec(strvec, i + 1, &val, 1, 255, false))
 				report_config_error(CONFIG_GENERAL_ERROR, "Invalid lvs_sync_daemon ttl (%s) - ignoring", strvec_slot(strvec, i+1));
 			else
 				global_data->lvs_syncd.mcast_ttl = (uint8_t)val;
@@ -499,6 +585,25 @@ lvs_syncd_handler(const vector_t *strvec)
 			continue;
 		}
 #endif
+
+		/* The following are for backward compatibility when lvs_sync_daemon IF VRRP_INSTANCE [SYNC_ID] could be specified */
+		if (i == 2) {
+			global_data->lvs_syncd.vrrp_name = STRDUP(strvec_slot(strvec, 2));
+			continue;
+		}
+
+		if (i == 3) {
+			if (!read_unsigned_strvec(strvec, 3, &val, 0, 255, false))
+				report_config_error(CONFIG_GENERAL_ERROR, "Invalid syncid (%s) - defaulting to vrid", strvec_slot(strvec, 3));
+			else {
+				report_config_error(CONFIG_GENERAL_ERROR, "Please use keyword \"id\" before lvs_sync_daemon syncid value");
+				global_data->lvs_syncd.syncid = val;
+			}
+
+			continue;
+		}
+
+		/* We haven't matched anything */
 		report_config_error(CONFIG_GENERAL_ERROR, "Unknown option %s specified for lvs_sync_daemon", strvec_slot(strvec, i));
 	}
 }
@@ -590,12 +695,23 @@ get_rt_rlimit(const vector_t *strvec, const char *process)
 {
 	unsigned limit;
 	rlim_t rlim;
+	size_t keyword_len;
+
+	/* *_rlimit_rtime is deprecated since 02/02/2020. Keyword should be *_rlimit_rttime */
+	keyword_len = strlen(strvec_slot(strvec, 0));
+	if (strvec_slot(strvec, 0)[keyword_len - 5] == 'r')
+		log_message(LOG_INFO, "Keyword '%s' is deprecated - please use '%.*srttime'", strvec_slot(strvec, 0), (int)keyword_len - 5, strvec_slot(strvec, 0));
 
 	if (!read_unsigned_strvec(strvec, 1, &limit, 1, UINT32_MAX, true)) {
 		report_config_error(CONFIG_GENERAL_ERROR, "Invalid %s real-time limit - %s", process, strvec_slot(strvec, 1));
 		return 0;
 	}
 
+	/* The rlim value is divided by 2 elsewhere, and the result must be
+	 * non-zero, therefore we need rlim to have a minimum value of 2. */
+	if (limit == 1)
+		limit = 2;
+
 	rlim = limit;
 	return rlim;
 }
@@ -643,10 +759,10 @@ vrrp_garp_delay_handler(const vector_t *strvec)
 {
 	unsigned timeout;
 
-        if (!read_unsigned_strvec(strvec, 1, &timeout, 0, UINT_MAX / TIMER_HZ, true)) {
+	if (!read_unsigned_strvec(strvec, 1, &timeout, 0, UINT_MAX / TIMER_HZ, true)) {
 		report_config_error(CONFIG_GENERAL_ERROR, "vrrp_garp_master_delay '%s' invalid - ignoring", strvec_slot(strvec, 1));
-                return;
-        }
+		return;
+	}
 
 	global_data->vrrp_garp_delay = timeout * TIMER_HZ;
 }
@@ -673,33 +789,33 @@ vrrp_garp_rep_handler(const vector_t *strvec)
 static void
 vrrp_garp_refresh_handler(const vector_t *strvec)
 {
-        unsigned refresh;
+	unsigned refresh;
 
-        if (!read_unsigned_strvec(strvec, 1, &refresh, 0, UINT_MAX, true)) {
-                report_config_error(CONFIG_GENERAL_ERROR, "Invalid vrrp_garp_master_refresh '%s' - ignoring", strvec_slot(strvec, 1));
-                global_data->vrrp_garp_refresh.tv_sec = 0;
-        }
-        else
+	if (!read_unsigned_strvec(strvec, 1, &refresh, 0, UINT_MAX, true)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "Invalid vrrp_garp_master_refresh '%s' - ignoring", strvec_slot(strvec, 1));
+		global_data->vrrp_garp_refresh.tv_sec = 0;
+	}
+	else
 		global_data->vrrp_garp_refresh.tv_sec = refresh;
 
-        global_data->vrrp_garp_refresh.tv_usec = 0;
+	global_data->vrrp_garp_refresh.tv_usec = 0;
 }
 static void
 vrrp_garp_refresh_rep_handler(const vector_t *strvec)
 {
-        unsigned repeats;
+	unsigned repeats;
 
-        /* The min value should be 1, but allow 0 to maintain backward compatibility
-         * with pre v2.0.7 */
-        if (!read_unsigned_strvec(strvec, 1, &repeats, 0, UINT_MAX, true)) {
-                report_config_error(CONFIG_GENERAL_ERROR, "vrrp_garp_master_refresh_repeat '%s' invalid - ignoring", strvec_slot(strvec, 1));
-                return;
-        }
+	/* The min value should be 1, but allow 0 to maintain backward compatibility
+	 * with pre v2.0.7 */
+	if (!read_unsigned_strvec(strvec, 1, &repeats, 0, UINT_MAX, true)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "vrrp_garp_master_refresh_repeat '%s' invalid - ignoring", strvec_slot(strvec, 1));
+		return;
+	}
 
-        if (repeats == 0) {
-                report_config_error(CONFIG_GENERAL_ERROR, "vrrp_garp_master_refresh_repeat must be greater than 0, setting to 1");
-                repeats = 1;
-        }
+	if (repeats == 0) {
+		report_config_error(CONFIG_GENERAL_ERROR, "vrrp_garp_master_refresh_repeat must be greater than 0, setting to 1");
+		repeats = 1;
+	}
 
 	global_data->vrrp_garp_refresh_rep = repeats;
 
@@ -707,12 +823,12 @@ vrrp_garp_refresh_rep_handler(const vector_t *strvec)
 static void
 vrrp_garp_lower_prio_delay_handler(const vector_t *strvec)
 {
-        unsigned delay;
+	unsigned delay;
 
-        if (!read_unsigned_strvec(strvec, 1, &delay, 0, UINT_MAX / TIMER_HZ, true)) {
-                report_config_error(CONFIG_GENERAL_ERROR, "vrrp_garp_lower_prio_delay '%s' invalid - ignoring", strvec_slot(strvec, 1));
-                return;
-        }
+	if (!read_unsigned_strvec(strvec, 1, &delay, 0, UINT_MAX / TIMER_HZ, true)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "vrrp_garp_lower_prio_delay '%s' invalid - ignoring", strvec_slot(strvec, 1));
+		return;
+	}
 
 	global_data->vrrp_garp_lower_prio_delay = delay * TIMER_HZ;
 }
@@ -721,10 +837,10 @@ vrrp_garp_lower_prio_rep_handler(const vector_t *strvec)
 {
 	unsigned garp_lower_prio_rep;
 
-        if (!read_unsigned_strvec(strvec, 1, &garp_lower_prio_rep, 0, INT_MAX, true)) {
-                report_config_error(CONFIG_GENERAL_ERROR, "Invalid vrrp_garp_lower_prio_repeat '%s'", strvec_slot(strvec, 1));
-                return;
-        }
+	if (!read_unsigned_strvec(strvec, 1, &garp_lower_prio_rep, 0, INT_MAX, true)) {
+		report_config_error(CONFIG_GENERAL_ERROR, "Invalid vrrp_garp_lower_prio_repeat '%s'", strvec_slot(strvec, 1));
+		return;
+	}
 
 	global_data->vrrp_garp_lower_prio_rep = garp_lower_prio_rep;
 }
@@ -815,22 +931,27 @@ vrrp_higher_prio_send_advert_handler(const vector_t *strvec)
 static void
 vrrp_iptables_handler(const vector_t *strvec)
 {
+	if (global_data->vrrp_iptables_inchain) {
+		report_config_error(CONFIG_GENERAL_ERROR, "iptables already specified - ignoring");
+		return;
+	}
+
 	if (vector_size(strvec) >= 2) {
-		if (strlen(strvec_slot(strvec,1)) >= sizeof(global_data->vrrp_iptables_inchain)-1) {
+		if (strlen(strvec_slot(strvec,1)) >= XT_EXTENSION_MAXNAMELEN - 1) {
 			report_config_error(CONFIG_GENERAL_ERROR, "VRRP Error : iptables in chain name too long - ignored");
 			return;
 		}
-		strcpy(global_data->vrrp_iptables_inchain, strvec_slot(strvec,1));
+		global_data->vrrp_iptables_inchain = STRDUP(strvec_slot(strvec,1));
 		if (vector_size(strvec) >= 3) {
-			if (strlen(strvec_slot(strvec,2)) >= sizeof(global_data->vrrp_iptables_outchain)-1) {
+			if (strlen(strvec_slot(strvec,2)) >= XT_EXTENSION_MAXNAMELEN - 1) {
 				report_config_error(CONFIG_GENERAL_ERROR, "VRRP Error : iptables out chain name too long - ignored");
 				return;
 			}
-			strcpy(global_data->vrrp_iptables_outchain, strvec_slot(strvec,2));
+			global_data->vrrp_iptables_outchain = STRDUP(strvec_slot(strvec,2));
 		}
 	} else {
-		strcpy(global_data->vrrp_iptables_inchain, DEFAULT_IPTABLES_CHAIN_IN);
-		strcpy(global_data->vrrp_iptables_outchain, DEFAULT_IPTABLES_CHAIN_OUT);
+		global_data->vrrp_iptables_inchain = STRDUP(DEFAULT_IPTABLES_CHAIN_IN);
+		global_data->vrrp_iptables_outchain = STRDUP(DEFAULT_IPTABLES_CHAIN_OUT);
 	}
 }
 #ifdef _HAVE_LIBIPSET_
@@ -838,74 +959,89 @@ static void
 vrrp_ipsets_handler(const vector_t *strvec)
 {
 	size_t len;
+	char set_name[IPSET_MAXNAMELEN];
 
-	if (vector_size(strvec) >= 2) {
-		if (strlen(strvec_slot(strvec,1)) >= sizeof(global_data->vrrp_ipset_address)-1) {
-			report_config_error(CONFIG_GENERAL_ERROR, "VRRP Error : ipset address name too long - ignored");
-			return;
-		}
-		strcpy(global_data->vrrp_ipset_address, strvec_slot(strvec,1));
-	}
-	else {
+	FREE_CONST_PTR(global_data->vrrp_ipset_address);
+	FREE_CONST_PTR(global_data->vrrp_ipset_address6);
+	FREE_CONST_PTR(global_data->vrrp_ipset_address_iface6);
+#ifdef HAVE_IPSET_ATTR_IFACE
+	FREE_CONST_PTR(global_data->vrrp_ipset_igmp);
+	FREE_CONST_PTR(global_data->vrrp_ipset_mld);
+#endif
+
+	if (vector_size(strvec) < 2) {
 		global_data->using_ipsets = false;
 		return;
 	}
 
+	if (strlen(strvec_slot(strvec,1)) >= IPSET_MAXNAMELEN - 1) {
+		report_config_error(CONFIG_GENERAL_ERROR, "VRRP Error : ipset address name too long - ignored");
+		return;
+	}
+	global_data->vrrp_ipset_address = STRDUP(strvec_slot(strvec,1));
+
 	if (vector_size(strvec) >= 3) {
-		if (strlen(strvec_slot(strvec,2)) >= sizeof(global_data->vrrp_ipset_address6)-1) {
+		if (strlen(strvec_slot(strvec,2)) >= IPSET_MAXNAMELEN - 1) {
 			report_config_error(CONFIG_GENERAL_ERROR, "VRRP Error : ipset IPv6 address name too long - ignored");
 			return;
 		}
-		strcpy(global_data->vrrp_ipset_address6, strvec_slot(strvec,2));
+		global_data->vrrp_ipset_address6 = STRDUP(strvec_slot(strvec,2));
 	}
 	else {
 		/* No second set specified, copy first name and add "6" */
-		strcpy(global_data->vrrp_ipset_address6, global_data->vrrp_ipset_address);
-		global_data->vrrp_ipset_address6[sizeof(global_data->vrrp_ipset_address6) - 2] = '\0';
-		strcat(global_data->vrrp_ipset_address6, "6");
+		strcpy_safe(set_name, global_data->vrrp_ipset_address);
+		set_name[IPSET_MAXNAMELEN - 2] = '\0';
+		strcat(set_name, "6");
+		global_data->vrrp_ipset_address6 = STRDUP(set_name);
 	}
 	if (vector_size(strvec) >= 4) {
-		if (strlen(strvec_slot(strvec,3)) >= sizeof(global_data->vrrp_ipset_address_iface6)-1) {
+		if (strlen(strvec_slot(strvec,3)) >= IPSET_MAXNAMELEN - 1) {
 			report_config_error(CONFIG_GENERAL_ERROR, "VRRP Error : ipset IPv6 address_iface name too long - ignored");
 			return;
 		}
-		strcpy(global_data->vrrp_ipset_address_iface6, strvec_slot(strvec,3));
+		global_data->vrrp_ipset_address_iface6 = STRDUP(strvec_slot(strvec,3));
 	}
 	else {
 		/* No third set specified, copy second name and add "_if6" */
-		strcpy(global_data->vrrp_ipset_address_iface6, global_data->vrrp_ipset_address6);
-		len = strlen(global_data->vrrp_ipset_address_iface6);
-		if (global_data->vrrp_ipset_address_iface6[len-1] == '6')
-			global_data->vrrp_ipset_address_iface6[--len] = '\0';
-		global_data->vrrp_ipset_address_iface6[sizeof(global_data->vrrp_ipset_address_iface6) - 5] = '\0';
-		strcat(global_data->vrrp_ipset_address_iface6, "_if6");
+		strcpy_safe(set_name, global_data->vrrp_ipset_address6);
+		len = strlen(set_name);
+		if (set_name[len-1] == '6')
+			set_name[--len] = '\0';
+		set_name[IPSET_MAXNAMELEN - 5] = '\0';
+		strcat(set_name, "_if6");
+		global_data->vrrp_ipset_address_iface6 = STRDUP(set_name);
 	}
+
+#ifdef HAVE_IPSET_ATTR_IFACE
 	if (vector_size(strvec) >= 5) {
-		if (strlen(strvec_slot(strvec,4)) >= sizeof(global_data->vrrp_ipset_igmp)-1) {
+		if (strlen(strvec_slot(strvec,4)) >= IPSET_MAXNAMELEN - 1) {
 			report_config_error(CONFIG_GENERAL_ERROR, "VRRP Error : ipset IGMP name too long - ignored");
 			return;
 		}
-		strcpy(global_data->vrrp_ipset_igmp, strvec_slot(strvec,4));
+		global_data->vrrp_ipset_igmp = STRDUP(strvec_slot(strvec,4));
 	}
 	else {
 		/* No second set specified, copy first name and add "_igmp" */
-		strcpy(global_data->vrrp_ipset_igmp, global_data->vrrp_ipset_address);
-		global_data->vrrp_ipset_address6[sizeof(global_data->vrrp_ipset_igmp) - 6] = '\0';
-		strcat(global_data->vrrp_ipset_igmp, "_igmp");
+		strcpy_safe(set_name, global_data->vrrp_ipset_address);
+		set_name[sizeof(set_name) - 6] = '\0';
+		strcat(set_name, "_igmp");
+		global_data->vrrp_ipset_igmp = STRDUP(set_name);
 	}
 	if (vector_size(strvec) >= 6) {
-		if (strlen(strvec_slot(strvec,5)) >= sizeof(global_data->vrrp_ipset_mld)-1) {
+		if (strlen(strvec_slot(strvec,5)) >= IPSET_MAXNAMELEN - 1) {
 			report_config_error(CONFIG_GENERAL_ERROR, "VRRP Error : ipset MLD name too long - ignored");
 			return;
 		}
-		strcpy(global_data->vrrp_ipset_mld, strvec_slot(strvec,5));
+		global_data->vrrp_ipset_mld = STRDUP(strvec_slot(strvec,5));
 	}
 	else {
 		/* No second set specified, copy first name and add "_mld" */
-		strcpy(global_data->vrrp_ipset_mld, global_data->vrrp_ipset_address);
-		global_data->vrrp_ipset_mld[sizeof(global_data->vrrp_ipset_mld) - 5] = '\0';
-		strcat(global_data->vrrp_ipset_mld, "_mld");
+		strcpy_safe(set_name, global_data->vrrp_ipset_address);
+		set_name[sizeof(set_name) - 5] = '\0';
+		strcat(set_name, "_mld");
+		global_data->vrrp_ipset_mld = STRDUP(set_name);
 	}
+#endif
 }
 #endif
 #endif
@@ -1278,6 +1414,24 @@ net_namespace_handler(const vector_t *strvec)
 		report_config_error(CONFIG_GENERAL_ERROR, "Duplicate net_namespace definition %s - ignoring", strvec_slot(strvec, 1));
 }
 
+static void
+net_namespace_ipvs_handler(const vector_t *strvec)
+{
+	if (!strvec)
+		return;
+
+	if (global_data->network_namespace_ipvs) {
+		report_config_error(CONFIG_GENERAL_ERROR, "Duplicate net_namespace_ipvs definition %s - ignoring", strvec_slot(strvec, 1));
+		return;
+	}
+
+	/* No namespace name means default namespace */
+	if (vector_size(strvec) < 2)
+		global_data->network_namespace_ipvs = STRDUP("");
+	else
+		global_data->network_namespace_ipvs = set_value(strvec);
+}
+
 static void
 namespace_ipsets_handler(const vector_t *strvec)
 {
@@ -1772,6 +1926,27 @@ random_seed_handler(const vector_t *strvec)
 	set_random_seed(val);
 }
 
+#ifndef _ONE_PROCESS_DEBUG_
+static void
+reload_time_file_handler(const vector_t *strvec)
+{
+	char *str;
+
+	if (vector_size(strvec) != 2) {
+		report_config_error(CONFIG_GENERAL_ERROR, "reload_time_file invalid");
+		return;
+	}
+	global_data->reload_time_file = str = MALLOC(strlen(strvec_slot(strvec, 1)) + 1);
+	strcpy(str, strvec_slot(strvec, 1));
+}
+
+static void
+reload_repeat_handler(__attribute__((unused)) const vector_t *strvec)
+{
+	global_data->reload_repeat = true;
+}
+#endif
+
 void
 init_global_keywords(bool global_active)
 {
@@ -1781,6 +1956,7 @@ init_global_keywords(bool global_active)
 #endif
 #if HAVE_DECL_CLONE_NEWNET
 	install_keyword_root("net_namespace", &net_namespace_handler, global_active);
+	install_keyword_root("net_namespace_ipvs", &net_namespace_ipvs_handler, global_active);
 	install_keyword_root("namespace_with_ipsets", &namespace_ipsets_handler, global_active);
 #endif
 	install_keyword_root("use_pid_dir", &use_pid_dir_handler, global_active);
@@ -1805,6 +1981,12 @@ init_global_keywords(bool global_active)
 	install_keyword("smtp_connect_timeout", &smtpto_handler);
 	install_keyword("notification_email", &email_handler);
 	install_keyword("smtp_alert", &smtp_alert_handler);
+	install_keyword("startup_script", &startup_script_handler);
+	install_keyword("startup_script_timeout", &startup_script_timeout_handler);
+	install_keyword("shutdown_script", &shutdown_script_handler);
+	install_keyword("shutdown_script_timeout", &shutdown_script_timeout_handler);
+	install_keyword("max_auto_priority", &max_auto_priority_handler);
+	install_keyword("min_auto_priority_delay", &min_auto_priority_delay_handler);
 #ifdef _WITH_VRRP_
 	install_keyword("smtp_alert_vrrp", &smtp_alert_vrrp_handler);
 #endif
@@ -1860,7 +2042,8 @@ init_global_keywords(bool global_active)
 	install_keyword("vrrp_rt_priority", &vrrp_rt_priority_handler);
 	install_keyword("vrrp_cpu_affinity", &vrrp_cpu_affinity_handler);
 #if HAVE_DECL_RLIMIT_RTTIME == 1
-	install_keyword("vrrp_rlimit_rtime", &vrrp_rt_rlimit_handler);
+	install_keyword("vrrp_rlimit_rttime", &vrrp_rt_rlimit_handler);
+	install_keyword("vrrp_rlimit_rtime", &vrrp_rt_rlimit_handler);		/* Deprecated 02/02/2020 */
 #endif
 #endif
 	install_keyword("notify_fifo", &global_notify_fifo);
@@ -1878,7 +2061,8 @@ init_global_keywords(bool global_active)
 	install_keyword("checker_rt_priority", &checker_rt_priority_handler);
 	install_keyword("checker_cpu_affinity", &checker_cpu_affinity_handler);
 #if HAVE_DECL_RLIMIT_RTTIME == 1
-	install_keyword("checker_rlimit_rtime", &checker_rt_rlimit_handler);
+	install_keyword("checker_rlimit_rttime", &checker_rt_rlimit_handler);
+	install_keyword("checker_rlimit_rtime", &checker_rt_rlimit_handler);	/* Deprecated 02/02/2020 */
 #endif
 #endif
 #ifdef _WITH_BFD_
@@ -1887,7 +2071,8 @@ init_global_keywords(bool global_active)
 	install_keyword("bfd_rt_priority", &bfd_rt_priority_handler);
 	install_keyword("bfd_cpu_affinity", &bfd_cpu_affinity_handler);
 #if HAVE_DECL_RLIMIT_RTTIME == 1
-	install_keyword("bfd_rlimit_rtime", &bfd_rt_rlimit_handler);
+	install_keyword("bfd_rlimit_rttime", &bfd_rt_rlimit_handler);
+	install_keyword("bfd_rlimit_rtime", &bfd_rt_rlimit_handler);		/* Deprecated 02/02/2020 */
 #endif
 #endif
 #ifdef _WITH_SNMP_
@@ -1944,4 +2129,8 @@ init_global_keywords(bool global_active)
 #endif
 	install_keyword("umask", &umask_handler);
 	install_keyword("random_seed", &random_seed_handler);
+#ifndef _ONE_PROCESS_DEBUG_
+	install_keyword("reload_time_file", &reload_time_file_handler);
+	install_keyword("reload_repeat", &reload_repeat_handler);
+#endif
 }
