@@ -17,23 +17,16 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #ifndef _DAEMON_H
 #define _DAEMON_H
 
 /* System includes */
-#include <stdio.h>
-#include <stdlib.h>
-#include <sys/stat.h>
-#include <sys/wait.h>
-#include <sys/types.h>
-#include <unistd.h>
-#include <errno.h>
-#include <fcntl.h>
+#include <stdbool.h>
 
 /* prototype */
-extern pid_t xdaemon(int, int, int);
+extern pid_t xdaemon(bool, bool, bool);
 
 #endif
