@@ -18,7 +18,7 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #ifndef _VRRP_NOTIFY_H
@@ -27,8 +27,10 @@
 /* local include */
 #include "vrrp.h"
 
-extern int notify_instance_exec(vrrp_t *, int);
-extern int notify_group_exec(vrrp_sgroup_t *, int);
-extern void notify_instance_fifo(const vrrp_t *, int);
+extern void send_event_notify(vrrp_t *, int);
+extern void send_instance_notifies(vrrp_t *);
+extern void send_group_notifies(vrrp_sgroup_t *);
+extern void send_instance_priority_notifies(vrrp_t *);
+extern void notify_shutdown(void);
 
 #endif
