@@ -65,12 +65,12 @@
 #ifdef _WITH_DBUS_
   #include "vrrp_dbus.h"
 #endif
-#include "list.h"
+#include "list_head.h"
 #include "main.h"
 #include "parser.h"
 #include "utils.h"
 #include "vrrp_notify.h"
-#include "vrrp_track.h"
+#include "track_file.h"
 #ifdef _WITH_JSON_
 #include "vrrp_json.h"
 #endif
@@ -83,17 +83,20 @@
 #ifdef _WITH_CN_PROC_
 #include "track_process.h"
 #endif
+#ifdef _WITH_LVS_
+#include "ipvswrapper.h"
+#endif
 
 /* Global variables */
 bool non_existent_interface_specified;
 
 /* Forward declarations */
 #ifndef _ONE_PROCESS_DEBUG_
-static int print_vrrp_data(thread_ref_t);
-static int print_vrrp_stats(thread_ref_t);
-static int reload_vrrp_thread(thread_ref_t);
+static void print_vrrp_data(thread_ref_t);
+static void print_vrrp_stats(thread_ref_t);
+static void reload_vrrp_thread(thread_ref_t);
 #ifdef _WITH_JSON_
-static int print_vrrp_json(thread_ref_t);
+static void print_vrrp_json(thread_ref_t);
 #endif
 #endif
 #ifdef _WITH_PERF_
@@ -102,8 +105,10 @@ perf_t perf_run = PERF_NONE;
 
 /* local variables */
 static const char *vrrp_syslog_ident;
-#ifndef __ONE_PROCESSDEBUG_
+#ifndef __ONE_PROCESS_DEBUG_
 static bool two_phase_terminate;
+static timeval_t vrrp_start_time;
+static unsigned vrrp_next_restart_delay;
 #endif
 
 #ifdef _VRRP_FD_DEBUG_
@@ -115,14 +120,13 @@ bool do_vrrp_fd_debug;
 static void
 dump_vrrp_fd(void)
 {
-	element e;
 	sock_t *sock;
 	vrrp_t *vrrp;
 	timeval_t time_diff;
 
 	log_message(LOG_INFO, "----[ Begin VRRP fd dump ]----");
 
-	LIST_FOREACH(vrrp_data->vrrp_socket_pool, sock, e) {
+	list_for_each_entry(sock, &vrrp_data->vrrp_socket_pool, e_list) {
 		log_message(LOG_INFO, "  Sockets %d, %d", sock->fd_in, sock->fd_out);
 
 		rb_for_each_entry_cached(vrrp, &sock->rb_sands, rb_sands) {
@@ -149,9 +153,16 @@ dump_vrrp_fd(void)
 static void
 set_vrrp_max_fds(void)
 {
-	if (!vrrp_data->vrrp)
+	vrrp_t *vrrp;
+	int cnt = 0;
+
+	if (list_empty(&vrrp_data->vrrp))
 		return;
 
+	/* This is called at boot so ok performing full walk */
+	list_for_each_entry(vrrp, &vrrp_data->vrrp, e_list)
+		cnt++;
+
 	/* Allow:
 	 * 2 per vrrp instance - always needed for VMAC instances
 	 *
@@ -177,14 +188,15 @@ set_vrrp_max_fds(void)
 	 *
 	 * 20 spare (in case we have forgotten anything)
 	 */
-	set_max_file_limit(LIST_SIZE(vrrp_data->vrrp) * 2 + vrrp_data->num_smtp_alert + 21 + 20);
+	set_max_file_limit(cnt * 2 + vrrp_data->num_smtp_alert + 21 + 20);
 }
 
 #ifdef _WITH_LVS_
 static bool
 vrrp_ipvs_needed(void)
 {
-	return !!(global_data->lvs_syncd.ifname);
+	return global_data->lvs_syncd.ifname &&
+	       (global_data->lvs_syncd.vrrp || global_data->lvs_syncd.vrrp_name);
 }
 #endif
 
@@ -276,7 +288,7 @@ vrrp_terminate_phase2(int exit_status)
 	FREE_CONST_PTR(vrrp_syslog_ident);
 #else
 	if (vrrp_syslog_ident)
-		free(no_const_char_p(vrrp_syslog_ident));
+		free(no_const_char_p(vrrp_syslog_ident));	/* malloc'd by make_syslog_ident() */
 #endif
 	close_std_fd();
 
@@ -286,7 +298,7 @@ vrrp_terminate_phase2(int exit_status)
 	exit(exit_status);
 }
 
-static int
+static void
 vrrp_shutdown_backstop_thread(thread_ref_t thread)
 {
 	int count = 0;
@@ -303,11 +315,9 @@ vrrp_shutdown_backstop_thread(thread_ref_t thread)
 			thread->master->shutdown_timer_running ? "" : "not ", count);
 
 	thread_add_terminate_event(thread->master);
-
-	return 0;
 }
 
-static int
+static void
 vrrp_shutdown_timer_thread(thread_ref_t thread)
 {
 	thread->master->shutdown_timer_running = false;
@@ -316,8 +326,6 @@ vrrp_shutdown_timer_thread(thread_ref_t thread)
 		thread_add_timer_shutdown(thread->master, vrrp_shutdown_backstop_thread, NULL, TIMER_HZ / 10);
 	else
 		thread_add_terminate_event(thread->master);
-
-	return 0;
 }
 
 /* Daemon stop sequence */
@@ -346,12 +354,12 @@ vrrp_terminate_phase1(bool schedule_next_thread)
 #endif
 
 #ifdef _WITH_LVS_
-        if (global_data->lvs_syncd.vrrp) {
-                /* Stop syncd if controlled by this VRRP instance. */
-                ipvs_syncd_cmd(IPVS_STOPDAEMON, &global_data->lvs_syncd,
-                               (global_data->lvs_syncd.vrrp->state == VRRP_STATE_MAST) ? IPVS_MASTER: IPVS_BACKUP,
-                               true, false);
-        }
+	if (vrrp_ipvs_needed()) {
+		/* Stop syncd if controlled by this vrrp process. */
+		ipvs_syncd_cmd(IPVS_STOPDAEMON, &global_data->lvs_syncd,
+			       global_data->lvs_syncd.vrrp->state == VRRP_STATE_MAST ? IPVS_MASTER: IPVS_BACKUP,
+			       false);
+	}
 #endif
 
 	/* Ensure any interfaces are in backup mode,
@@ -365,15 +373,15 @@ vrrp_terminate_phase1(bool schedule_next_thread)
 		report_and_clear_netlink_timers("Restored interfaces");
 #endif
 
-	if (vrrp_data->vrrp_track_files)
+	if (!list_empty(&vrrp_data->vrrp_track_files))
 		stop_track_files();
 
 	/* Clear static entries */
 #ifdef _HAVE_FIB_ROUTING_
-	netlink_rulelist(vrrp_data->static_rules, IPRULE_DEL, false);
-	netlink_rtlist(vrrp_data->static_routes, IPROUTE_DEL);
+	netlink_rulelist(&vrrp_data->static_rules, IPRULE_DEL, false);
+	netlink_rtlist(&vrrp_data->static_routes, IPROUTE_DEL);
 #endif
-	netlink_iplist(vrrp_data->static_addresses, IPADDRESS_DEL, false);
+	netlink_iplist(&vrrp_data->static_addresses, IPADDRESS_DEL, false);
 
 #ifdef _NETLINK_TIMERS_
 	if (do_netlink_timers)
@@ -387,7 +395,7 @@ vrrp_terminate_phase1(bool schedule_next_thread)
 	notify_shutdown();
 
 	if (schedule_next_thread) {
-		if (!LIST_ISEMPTY(vrrp_data->vrrp)) {
+		if (!list_empty(&vrrp_data->vrrp)) {
 			/* This is not nice, but it significantly increases the chances
 			 * of an IGMP leave group being sent for some reason.
 			 * Since we are about to exit, it doesn't affect anything else
@@ -405,15 +413,13 @@ vrrp_terminate_phase1(bool schedule_next_thread)
 }
 
 #ifndef _ONE_PROCESS_DEBUG_
-static int
+static void
 start_vrrp_termination_thread(__attribute__((unused)) thread_ref_t thread)
 {
 	/* This runs in the context of a thread */
 	two_phase_terminate = true;
 
 	vrrp_terminate_phase1(true);
-
-	return 0;
 }
 #endif
 
@@ -446,13 +452,6 @@ start_vrrp(data_t *prev_global_data)
 
 	if (!global_data)
 		global_data = alloc_global_data();
-	else if (global_data->default_ifname) {
-		/* We need to set the default_ifp here on startup, since
-		 * the parent process doesn't know about the interfaces */
-		global_data->default_ifp = if_get_by_ifname(global_data->default_ifname, IF_CREATE_IF_DYNAMIC);
-		if (!global_data->default_ifp)
-			log_message(LOG_INFO, "WARNING - default interface %s doesn't exist", global_data->default_ifname);
-	}
 
 	/* Parse configuration file */
 	vrrp_data = alloc_vrrp_data();
@@ -484,7 +483,7 @@ start_vrrp(data_t *prev_global_data)
 
 	if (!__test_bit(CONFIG_TEST_BIT, &debug)) {
 #if defined _WITH_SNMP_RFC_ || defined _WITH_SNMP_VRRP_
-		if (!reload && (
+		if ((
 #ifdef _WITH_SNMP_VRRP_
 		     global_data->enable_snmp_vrrp ||
 #endif
@@ -495,9 +494,12 @@ start_vrrp(data_t *prev_global_data)
 		     global_data->enable_snmp_rfcv3 ||
 #endif
 		     snmp_option)) {
-			vrrp_snmp_agent_init(global_data->snmp_socket);
+			if (reload)
+				snmp_epoll_info(master);
+			else
+				vrrp_snmp_agent_init(global_data->snmp_socket);
 #ifdef _WITH_SNMP_RFC_
-			vrrp_start_time = time_now;
+			snmp_vrrp_start_time = time_now;
 #endif
 		}
 #endif
@@ -509,18 +511,6 @@ start_vrrp(data_t *prev_global_data)
 				stop_vrrp(KEEPALIVED_EXIT_FATAL);
 				return;
 			}
-
-			/* Set LVS timeouts */
-			if (global_data->lvs_tcp_timeout ||
-			    global_data->lvs_tcpfin_timeout ||
-			    global_data->lvs_udp_timeout)
-				ipvs_set_timeouts(global_data->lvs_tcp_timeout, global_data->lvs_tcpfin_timeout, global_data->lvs_udp_timeout);
-
-			/* If we are managing the sync daemon, then stop any
-			 * instances of it that may have been running if
-			 * we terminated abnormally */
-			ipvs_syncd_cmd(IPVS_STOPDAEMON, NULL, IPVS_MASTER, true, true);
-			ipvs_syncd_cmd(IPVS_STOPDAEMON, NULL, IPVS_BACKUP, true, true);
 		}
 #endif
 
@@ -528,10 +518,10 @@ start_vrrp(data_t *prev_global_data)
 			kernel_netlink_set_recv_bufs();
 
 #ifdef _HAVE_FIB_ROUTING_
-			clear_diff_srules();
-			clear_diff_sroutes();
+			clear_diff_static_rules();
+			clear_diff_static_routes();
 #endif
-			clear_diff_saddresses();
+			clear_diff_static_addresses();
 			clear_diff_script();
 #ifdef _WITH_BFD_
 			clear_diff_bfd();
@@ -539,11 +529,11 @@ start_vrrp(data_t *prev_global_data)
 		}
 		else {
 			/* Clear leftover static entries */
-			netlink_iplist(vrrp_data->static_addresses, IPADDRESS_DEL, false);
+			netlink_iplist(&vrrp_data->static_addresses, IPADDRESS_DEL, false);
 #ifdef _HAVE_FIB_ROUTING_
-			netlink_rtlist(vrrp_data->static_routes, IPROUTE_DEL);
+			netlink_rtlist(&vrrp_data->static_routes, IPROUTE_DEL);
 			netlink_error_ignore = ENOENT;
-			netlink_rulelist(vrrp_data->static_rules, IPRULE_DEL, true);
+			netlink_rulelist(&vrrp_data->static_rules, IPRULE_DEL, true);
 			netlink_error_ignore = 0;
 #endif
 		}
@@ -582,7 +572,7 @@ start_vrrp(data_t *prev_global_data)
 #ifdef _WITH_DBUS_
 	if (global_data->enable_dbus) {
 		if (reload && old_global_data->enable_dbus)
-			dbus_reload(old_vrrp_data->vrrp, vrrp_data->vrrp);
+			dbus_reload(&old_vrrp_data->vrrp, &vrrp_data->vrrp);
 		else {
 			if (!dbus_start())
 				global_data->enable_dbus = false;
@@ -598,10 +588,10 @@ start_vrrp(data_t *prev_global_data)
 #endif
 
 	/* Set static entries */
-	netlink_iplist(vrrp_data->static_addresses, IPADDRESS_ADD, false);
+	netlink_iplist(&vrrp_data->static_addresses, IPADDRESS_ADD, false);
 #ifdef _HAVE_FIB_ROUTING_
-	netlink_rtlist(vrrp_data->static_routes, IPROUTE_ADD);
-	netlink_rulelist(vrrp_data->static_rules, IPRULE_ADD, false);
+	netlink_rtlist(&vrrp_data->static_routes, IPROUTE_ADD);
+	netlink_rulelist(&vrrp_data->static_rules, IPRULE_ADD, false);
 #endif
 
 	/* Dump configuration */
@@ -617,7 +607,7 @@ start_vrrp(data_t *prev_global_data)
 		thread_add_event(master, vrrp_dispatcher_init, NULL, 0);
 
 	/* Set the process priority and non swappable if configured */
-	set_process_priorities(global_data->vrrp_realtime_priority,
+	set_process_priorities(global_data->vrrp_realtime_priority, global_data->max_auto_priority, global_data->min_auto_priority_delay,
 #if HAVE_DECL_RLIMIT_RTTIME == 1
 			       global_data->vrrp_rlimit_rt,
 #endif
@@ -631,7 +621,7 @@ start_vrrp(data_t *prev_global_data)
 }
 
 #ifndef _ONE_PROCESS_DEBUG_
-static int
+static void
 send_reload_advert_thread(thread_ref_t thread)
 {
 	vrrp_t *vrrp = THREAD_ARG(thread);
@@ -643,29 +633,24 @@ send_reload_advert_thread(thread_ref_t thread)
 	 * actual reload. */
 	if (THREAD_VAL(thread))
 		thread_add_event(master, reload_vrrp_thread, NULL, 0);
-
-	return 0;
 }
 
 static void
 sigreload_vrrp(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
 {
-	element e;
 	vrrp_t *vrrp;
 	int num_master_inst = 0;
-	int i;
+	int i = 0;
 
 	/* We want to send adverts for the vrrp instances which are
 	 * in master state. After that the reload can be initiated */
-	if (!LIST_ISEMPTY(vrrp_data->vrrp)) {
-		for (e = LIST_HEAD(vrrp_data->vrrp); e; ELEMENT_NEXT(e)) {
-			vrrp = ELEMENT_DATA(e);
+	if (!list_empty(&vrrp_data->vrrp)) {
+		list_for_each_entry(vrrp, &vrrp_data->vrrp, e_list) {
 			if (vrrp->state == VRRP_STATE_MAST)
 				num_master_inst++;
 		}
 
-		for (e = LIST_HEAD(vrrp_data->vrrp), i = 0; e; ELEMENT_NEXT(e)) {
-			vrrp = ELEMENT_DATA(e);
+		list_for_each_entry(vrrp, &vrrp_data->vrrp, e_list) {
 			if (vrrp->state == VRRP_STATE_MAST) {
 				i++;
 				thread_add_event(master, send_reload_advert_thread, vrrp, i == num_master_inst);
@@ -686,11 +671,11 @@ sigusr1_vrrp(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
 }
 
 static void
-sigusr2_vrrp(__attribute__((unused)) void *v, __attribute__((unused)) int sig)
+sigusr2_vrrp(__attribute__((unused)) void *v, int sig)
 {
-	log_message(LOG_INFO, "Printing VRRP stats for process(%d) on signal",
-		    getpid());
-	thread_add_event(master, print_vrrp_stats, NULL, 0);
+	log_message(LOG_INFO, "Printing %sVRRP stats for process(%d) on signal",
+		    sig == SIGSTATS_CLEAR ? "and clearing " : "", getpid());
+	thread_add_event(master, print_vrrp_stats, NULL, sig);
 }
 
 #ifdef _WITH_JSON_
@@ -720,6 +705,7 @@ vrrp_signal_init(void)
 	signal_set(SIGTERM, sigend_vrrp, NULL);
 	signal_set(SIGUSR1, sigusr1_vrrp, NULL);
 	signal_set(SIGUSR2, sigusr2_vrrp, NULL);
+	signal_set(SIGSTATS_CLEAR, sigusr2_vrrp, NULL);
 #ifdef _WITH_JSON_
 	signal_set(SIGJSON, sigjson_vrrp, NULL);
 #endif
@@ -730,10 +716,13 @@ vrrp_signal_init(void)
 }
 
 /* Reload thread */
-static int
+static void
 reload_vrrp_thread(__attribute__((unused)) thread_ref_t thread)
 {
 	bool with_snmp = false;
+#ifdef _WITH_LVS_
+	bool want_syncd_master;
+#endif
 
 	log_message(LOG_INFO, "Reloading");
 
@@ -746,7 +735,7 @@ reload_vrrp_thread(__attribute__((unused)) thread_ref_t thread)
 	/* Terminate all script process */
 	script_killall(master, SIGTERM, false);
 
-	if (vrrp_data->vrrp_track_files)
+	if (!list_empty(&vrrp_data->vrrp_track_files))
 		stop_track_files();
 
 	vrrp_initialised = false;
@@ -774,14 +763,6 @@ reload_vrrp_thread(__attribute__((unused)) thread_ref_t thread)
 	thread_cleanup_master(master);
 	thread_add_base_threads(master, with_snmp);
 
-#ifdef _WITH_LVS_
-	if (global_data->lvs_syncd.ifname)
-		ipvs_syncd_cmd(IPVS_STOPDAEMON, &global_data->lvs_syncd,
-		       (global_data->lvs_syncd.vrrp->state == VRRP_STATE_MAST) ? IPVS_MASTER:
-										 IPVS_BACKUP,
-		       true, false);
-#endif
-
 	/* Remove the notify fifo - we don't know if it will be the same after a reload */
 	notify_fifo_close(&global_data->notify_fifo, &global_data->vrrp_notify_fifo);
 
@@ -806,11 +787,13 @@ reload_vrrp_thread(__attribute__((unused)) thread_ref_t thread)
 	start_vrrp(old_global_data);
 
 #ifdef _WITH_LVS_
-	if (global_data->lvs_syncd.ifname)
-		ipvs_syncd_cmd(IPVS_STARTDAEMON, &global_data->lvs_syncd,
-			       (global_data->lvs_syncd.vrrp->state == VRRP_STATE_MAST) ? IPVS_MASTER:
-											 IPVS_BACKUP,
-			       true, false);
+	if (vrrp_ipvs_needed()) {
+		want_syncd_master = (global_data->lvs_syncd.vrrp->state == VRRP_STATE_MAST);
+		if (ipvs_syncd_changed(&old_global_data->lvs_syncd, &global_data->lvs_syncd))
+			ipvs_syncd_cmd(IPVS_STOPDAEMON, NULL, want_syncd_master ? IPVS_MASTER : IPVS_BACKUP, true);
+		ipvs_syncd_cmd(IPVS_STARTDAEMON, NULL, want_syncd_master ? IPVS_MASTER : IPVS_BACKUP, true);
+		ipvs_syncd_cmd(IPVS_STOPDAEMON, NULL, want_syncd_master ? IPVS_BACKUP : IPVS_MASTER, true);
+	}
 #endif
 
 	/* free backup data */
@@ -820,48 +803,57 @@ reload_vrrp_thread(__attribute__((unused)) thread_ref_t thread)
 	free_old_interface_queue();
 
 	UNSET_RELOAD;
-
-	return 0;
 }
 
-static int
+static void
 print_vrrp_data(__attribute__((unused)) thread_ref_t thread)
 {
 	vrrp_print_data();
-	return 0;
 }
 
-static int
-print_vrrp_stats(__attribute__((unused)) thread_ref_t thread)
+static void
+print_vrrp_stats(thread_ref_t thread)
 {
-	vrrp_print_stats();
-	return 0;
+	vrrp_print_stats(thread->u.val == SIGSTATS_CLEAR);
 }
 
 #ifdef _WITH_JSON_
-static int
+static void
 print_vrrp_json(__attribute__((unused)) thread_ref_t thread)
 {
 	vrrp_print_json();
-	return 0;
 }
 #endif
 
-/* VRRP Child respawning thread */
-static int
+/* This function runs in the parent process. */
+static void
+delayed_restart_vrrp_child_thread(__attribute__((unused)) thread_ref_t thread)
+{
+	start_vrrp_child();
+}
+
+/* VRRP Child respawning thread. This function runs in the parent process. */
+static void
 vrrp_respawn_thread(thread_ref_t thread)
 {
+	unsigned restart_delay;
+
 	/* We catch a SIGCHLD, handle it */
 	vrrp_child = 0;
 
-	if (!__test_bit(DONT_RESPAWN_BIT, &debug)) {
+	if (report_child_status(thread->u.c.status, thread->u.c.pid, NULL))
+		thread_add_terminate_event(thread->master);
+	else if (!__test_bit(DONT_RESPAWN_BIT, &debug)) {
 		log_message(LOG_ALERT, "VRRP child process(%d) died: Respawning", thread->u.c.pid);
-		start_vrrp_child();
+		restart_delay = calc_restart_delay(&vrrp_start_time, &vrrp_next_restart_delay, "VRRP");
+		if (!restart_delay)
+			start_vrrp_child();
+		else
+			thread_add_timer(thread->master, delayed_restart_vrrp_child_thread, NULL, restart_delay * TIMER_HZ);
 	} else {
 		log_message(LOG_ALERT, "VRRP child process(%d) died: Exiting", thread->u.c.pid);
 		raise(SIGTERM);
 	}
-	return 0;
 }
 #endif
 
@@ -869,6 +861,9 @@ vrrp_respawn_thread(thread_ref_t thread)
 static void
 register_vrrp_thread_addresses(void)
 {
+	/* Remove anything we might have inherited from parent */
+	deregister_thread_addresses();
+
 	register_scheduler_addresses();
 	register_signal_thread_addresses();
 	register_notify_addresses();
@@ -885,7 +880,7 @@ register_vrrp_thread_addresses(void)
 	register_vrrp_dbus_addresses();
 #endif
 	register_vrrp_fifo_addresses();
-	register_vrrp_inotify_addresses();
+	register_track_file_inotify_addresses();
 #ifdef _WITH_CN_PROC_
 	register_process_monitor_addresses();
 #endif
@@ -934,6 +929,8 @@ start_vrrp_child(void)
 		return -1;
 	} else if (pid) {
 		vrrp_child = pid;
+		vrrp_start_time = time_now;
+
 		log_message(LOG_INFO, "Starting VRRP child process, pid=%d"
 			       , pid);
 
@@ -1069,6 +1066,7 @@ register_vrrp_parent_addresses(void)
 {
 #ifndef _ONE_PROCESS_DEBUG_
 	register_thread_address("vrrp_respawn_thread", vrrp_respawn_thread);
+	register_thread_address("delayed_restart_vrrp_child_thread", delayed_restart_vrrp_child_thread);
 #endif
 }
 #endif
