@@ -24,6 +24,9 @@
 
 /* system includes */
 #include <unistd.h>
+#ifdef _HAVE_LINUX_IF_ETHER_H_COLLISION_
+#include <netinet/in.h>
+#endif
 #include <net/ethernet.h>
 #include <linux/if_packet.h>
 #include <netinet/icmp6.h>
@@ -235,8 +238,8 @@ ndisc_send_unsolicited_na(vrrp_t *vrrp, ip_address_t *ipaddress)
 	interface_t *ifp = IF_BASE_IFP(ipaddress->ifp);
 
 	/* If the interface doesn't support NDISC, don't try sending */
-        if (ifp->ifi_flags & IFF_NOARP)
-                return;
+	if (ifp->ifi_flags & IFF_NOARP)
+		return;
 
 	set_time_now();
 
