@@ -18,13 +18,17 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2016 Alexandre Cassen, <acassen@linux-vs.org>
+ * Copyright (C) 2001-2017 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #ifndef _READ_RTTABLES_H
 #define _READ_RTTABLES_H
 
+#include <stdbool.h>
+#include <stdint.h>
+
 extern void clear_rt_names(void);
+#ifdef _HAVE_FIB_ROUTING_
 extern bool find_rttables_table(const char *, uint32_t *);
 extern bool find_rttables_dsfield(const char *, uint8_t *);
 extern bool find_rttables_realms(const char *, uint32_t *);
@@ -32,12 +36,16 @@ extern bool find_rttables_realms(const char *, uint32_t *);
 extern bool find_rttables_group(const char *, uint32_t *);
 #endif
 extern bool find_rttables_proto(const char *, uint8_t *);
-extern bool find_rttables_scope(const char *, uint8_t *);
 extern bool find_rttables_rtntype(const char *, uint8_t *);
+#endif
+extern bool find_rttables_scope(const char *, uint8_t *);
+
 extern const char *get_rttables_scope(uint32_t);
+#ifdef _HAVE_FIB_ROUTING_
 #if HAVE_DECL_FRA_SUPPRESS_IFGROUP
 extern const char *get_rttables_group(uint32_t);
 #endif
 extern const char *get_rttables_rtntype(uint8_t);
+#endif
 
 #endif
